from __future__ import unicode_literals
import json
import base64

from moto.core.responses import BaseResponse
from .models import acm_backends, AWSError, AWSValidationException


class AWSCertificateManagerResponse(BaseResponse):

    @property
    def acm_backend(self):
        """
        ACM Backend

        :return: ACM Backend object
        :rtype: moto.acm.models.AWSCertificateManagerBackend
        """
        return acm_backends[self.region]

    @property
    def request_params(self):
        try:
            return json.loads(self.body)
        except ValueError:
            return {}

    def _get_param(self, param, default=None):
        return self.request_params.get(param, default)

    def add_tags_to_certificate(self):
        arn = self._get_param('CertificateArn')
        tags = self._get_param('Tags')

        if arn is None:
            msg = 'A required parameter for the specified action is not supplied.'
            return json.dumps({'__type': 'MissingParameter', 'message': msg}), dict(status=400)

        try:
            self.acm_backend.add_tags_to_certificate(arn, tags)
        except AWSError as err:
            return err.response()

        return ''

    def delete_certificate(self):
        arn = self._get_param('CertificateArn')

        if arn is None:
            msg = 'A required parameter for the specified action is not supplied.'
            return json.dumps({'__type': 'MissingParameter', 'message': msg}), dict(status=400)

        try:
            self.acm_backend.delete_certificate(arn)
        except AWSError as err:
            return err.response()

        return ''

    def describe_certificate(self):
        arn = self._get_param('CertificateArn')

        if arn is None:
            msg = 'A required parameter for the specified action is not supplied.'
            return json.dumps({'__type': 'MissingParameter', 'message': msg}), dict(status=400)

        try:
            cert_bundle = self.acm_backend.get_certificate(arn)
        except AWSError as err:
            return err.response()

        return json.dumps(cert_bundle.describe())

    def get_certificate(self):
        arn = self._get_param('CertificateArn')

        if arn is None:
            msg = 'A required parameter for the specified action is not supplied.'
            return json.dumps({'__type': 'MissingParameter', 'message': msg}), dict(status=400)

        try:
            cert_bundle = self.acm_backend.get_certificate(arn)
        except AWSError as err:
            return err.response()

        result = {
            'Certificate': cert_bundle.cert.decode(),
            'CertificateChain': cert_bundle.chain.decode()
        }
        return json.dumps(result)

    def import_certificate(self):
        """
        Returns errors on:
        Certificate, PrivateKey or Chain not being properly formatted
        Arn not existing if its provided
        PrivateKey size > 2048
        Certificate expired or is not yet in effect

        Does not return errors on:
        Checking Certificate is legit, or a selfsigned chain is provided

        :return: str(JSON) for response
        """
        certificate = self._get_param('Certificate')
        private_key = self._get_param('PrivateKey')
        chain = self._get_param('CertificateChain')  # Optional
        current_arn = self._get_param('CertificateArn')  # Optional

        # Simple parameter decoding. Rather do it here as its a data transport decision not part of the
        # actual data
        try:
            certificate = base64.standard_b64decode(certificate)
        except:
            return AWSValidationException('The certificate is not PEM-encoded or is not valid.').response()
        try:
            private_key = base64.standard_b64decode(private_key)
        except:
            return AWSValidationException('The private key is not PEM-encoded or is not valid.').response()
        if chain is not None:
            try:
                chain = base64.standard_b64decode(chain)
            except:
                return AWSValidationException('The certificate chain is not PEM-encoded or is not valid.').response()

        try:
            arn = self.acm_backend.import_cert(certificate, private_key, chain=chain, arn=current_arn)
        except AWSError as err:
            return err.response()

        return json.dumps({'CertificateArn': arn})

    def list_certificates(self):
        certs = []

        for cert_bundle in self.acm_backend.get_certificates_list():
            certs.append({
                'CertificateArn': cert_bundle.arn,
                'DomainName': cert_bundle.common_name
            })

        result = {'CertificateSummaryList': certs}
        return json.dumps(result)

    def list_tags_for_certificate(self):
        arn = self._get_param('CertificateArn')

        if arn is None:
            msg = 'A required parameter for the specified action is not supplied.'
            return {'__type': 'MissingParameter', 'message': msg}, dict(status=400)

        try:
            cert_bundle = self.acm_backend.get_certificate(arn)
        except AWSError as err:
            return err.response()

        result = {'Tags': []}
        # Tag "objects" can not contain the Value part
        for key, value in cert_bundle.tags.items():
            tag_dict = {'Key': key}
            if value is not None:
                tag_dict['Value'] = value
            result['Tags'].append(tag_dict)

        return json.dumps(result)

    def remove_tags_from_certificate(self):
        arn = self._get_param('CertificateArn')
        tags = self._get_param('Tags')

        if arn is None:
            msg = 'A required parameter for the specified action is not supplied.'
            return json.dumps({'__type': 'MissingParameter', 'message': msg}), dict(status=400)

        try:
            self.acm_backend.remove_tags_from_certificate(arn, tags)
        except AWSError as err:
            return err.response()

        return ''

    def request_certificate(self):
        domain_name = self._get_param('DomainName')
        domain_validation_options = self._get_param('DomainValidationOptions')  # is ignored atm
        idempotency_token = self._get_param('IdempotencyToken')
        subject_alt_names = self._get_param('SubjectAlternativeNames')

        if len(subject_alt_names) > 10:
            # There is initial AWS limit of 10
            msg = 'An ACM limit has been exceeded. Need to request SAN limit to be raised'
            return json.dumps({'__type': 'LimitExceededException', 'message': msg}), dict(status=400)

        try:
            arn = self.acm_backend.request_certificate(domain_name, domain_validation_options, idempotency_token, subject_alt_names)
        except AWSError as err:
            return err.response()

        return json.dumps({'CertificateArn': arn})

    def resend_validation_email(self):
        arn = self._get_param('CertificateArn')
        domain = self._get_param('Domain')
        # ValidationDomain not used yet.
        # Contains domain which is equal to or a subset of Domain
        # that AWS will send validation emails to
        # https://docs.aws.amazon.com/acm/latest/APIReference/API_ResendValidationEmail.html
        # validation_domain = self._get_param('ValidationDomain')

        if arn is None:
            msg = 'A required parameter for the specified action is not supplied.'
            return json.dumps({'__type': 'MissingParameter', 'message': msg}), dict(status=400)

        try:
            cert_bundle = self.acm_backend.get_certificate(arn)

            if cert_bundle.common_name != domain:
                msg = 'Parameter Domain does not match certificate domain'
                _type = 'InvalidDomainValidationOptionsException'
                return json.dumps({'__type': _type, 'message': msg}), dict(status=400)

        except AWSError as err:
            return err.response()

        return ''



from __future__ import unicode_literals
import six
import random


def create_id():
    size = 10
    chars = list(range(10)) + ['A-Z']
    return ''.join(six.text_type(random.choice(chars)) for x in range(size))



from __future__ import unicode_literals, print_function

import six
import boto
import boto3
import sure  # noqa
import requests
from moto import mock_dynamodb2, mock_dynamodb2_deprecated
from moto.dynamodb2 import dynamodb_backend2
from boto.exception import JSONResponseError
from botocore.exceptions import ClientError
from boto3.dynamodb.conditions import Key
from tests.helpers import requires_boto_gte
import tests.backport_assert_raises
from nose.tools import assert_raises
try:
    import boto.dynamodb2
except ImportError:
    print("This boto version is not supported")


@requires_boto_gte("2.9")
@mock_dynamodb2_deprecated
def test_list_tables():
    name = 'TestTable'
    #{'schema': }
    dynamodb_backend2.create_table(name, schema=[
        {u'KeyType': u'HASH', u'AttributeName': u'forum_name'},
        {u'KeyType': u'RANGE', u'AttributeName': u'subject'}
    ])
    conn = boto.dynamodb2.connect_to_region(
        'us-west-2',
        aws_access_key_id="ak",
        aws_secret_access_key="sk")
    assert conn.list_tables()["TableNames"] == [name]


@requires_boto_gte("2.9")
@mock_dynamodb2_deprecated
def test_list_tables_layer_1():
    dynamodb_backend2.create_table("test_1", schema=[
        {u'KeyType': u'HASH', u'AttributeName': u'name'}
    ])
    dynamodb_backend2.create_table("test_2", schema=[
        {u'KeyType': u'HASH', u'AttributeName': u'name'}
    ])
    conn = boto.dynamodb2.connect_to_region(
        'us-west-2',
        aws_access_key_id="ak",
        aws_secret_access_key="sk")

    res = conn.list_tables(limit=1)
    expected = {"TableNames": ["test_1"], "LastEvaluatedTableName": "test_1"}
    res.should.equal(expected)

    res = conn.list_tables(limit=1, exclusive_start_table_name="test_1")
    expected = {"TableNames": ["test_2"]}
    res.should.equal(expected)


@requires_boto_gte("2.9")
@mock_dynamodb2_deprecated
def test_describe_missing_table():
    conn = boto.dynamodb2.connect_to_region(
        'us-west-2',
        aws_access_key_id="ak",
        aws_secret_access_key="sk")
    with assert_raises(JSONResponseError):
        conn.describe_table('messages')


@requires_boto_gte("2.9")
@mock_dynamodb2
def test_list_table_tags():
    name = 'TestTable'
    conn = boto3.client('dynamodb',
                        region_name='us-west-2',
                        aws_access_key_id="ak",
                        aws_secret_access_key="sk")
    conn.create_table(TableName=name,
                      KeySchema=[{'AttributeName':'id','KeyType':'HASH'}],
                      AttributeDefinitions=[{'AttributeName':'id','AttributeType':'S'}],
                      ProvisionedThroughput={'ReadCapacityUnits':5,'WriteCapacityUnits':5})
    table_description = conn.describe_table(TableName=name)
    arn = table_description['Table']['TableArn']
    tags = [{'Key':'TestTag', 'Value': 'TestValue'}]
    conn.tag_resource(ResourceArn=arn,
                      Tags=tags)
    resp = conn.list_tags_of_resource(ResourceArn=arn)
    assert resp["Tags"] == tags


@requires_boto_gte("2.9")
@mock_dynamodb2
def test_list_table_tags_empty():
    name = 'TestTable'
    conn = boto3.client('dynamodb',
                        region_name='us-west-2',
                        aws_access_key_id="ak",
                        aws_secret_access_key="sk")
    conn.create_table(TableName=name,
                      KeySchema=[{'AttributeName':'id','KeyType':'HASH'}],
                      AttributeDefinitions=[{'AttributeName':'id','AttributeType':'S'}],
                      ProvisionedThroughput={'ReadCapacityUnits':5,'WriteCapacityUnits':5})
    table_description = conn.describe_table(TableName=name)
    arn = table_description['Table']['TableArn']
    tags = [{'Key':'TestTag', 'Value': 'TestValue'}]
    # conn.tag_resource(ResourceArn=arn,
    #                   Tags=tags)
    resp = conn.list_tags_of_resource(ResourceArn=arn)
    assert resp["Tags"] == []


@requires_boto_gte("2.9")
@mock_dynamodb2
def test_list_table_tags_paginated():
    name = 'TestTable'
    conn = boto3.client('dynamodb',
                        region_name='us-west-2',
                        aws_access_key_id="ak",
                        aws_secret_access_key="sk")
    conn.create_table(TableName=name,
                      KeySchema=[{'AttributeName':'id','KeyType':'HASH'}],
                      AttributeDefinitions=[{'AttributeName':'id','AttributeType':'S'}],
                      ProvisionedThroughput={'ReadCapacityUnits':5,'WriteCapacityUnits':5})
    table_description = conn.describe_table(TableName=name)
    arn = table_description['Table']['TableArn']
    for i in range(11):
        tags = [{'Key':'TestTag%d' % i, 'Value': 'TestValue'}]
        conn.tag_resource(ResourceArn=arn,
                          Tags=tags)
    resp = conn.list_tags_of_resource(ResourceArn=arn)
    assert len(resp["Tags"]) == 10
    assert 'NextToken' in resp.keys()
    resp2 = conn.list_tags_of_resource(ResourceArn=arn,
                                       NextToken=resp['NextToken'])
    assert len(resp2["Tags"]) == 1
    assert 'NextToken' not in resp2.keys()


@requires_boto_gte("2.9")
@mock_dynamodb2
def test_list_not_found_table_tags():
    conn = boto3.client('dynamodb',
                        region_name='us-west-2',
                        aws_access_key_id="ak",
                        aws_secret_access_key="sk")
    arn = 'DymmyArn'
    try:
        conn.list_tags_of_resource(ResourceArn=arn)
    except ClientError as exception:
        assert exception.response['Error']['Code'] == "ResourceNotFoundException"


@requires_boto_gte("2.9")
@mock_dynamodb2
def test_item_add_empty_string_exception():
    name = 'TestTable'
    conn = boto3.client('dynamodb',
                        region_name='us-west-2',
                        aws_access_key_id="ak",
                        aws_secret_access_key="sk")
    conn.create_table(TableName=name,
                      KeySchema=[{'AttributeName':'forum_name','KeyType':'HASH'}],
                      AttributeDefinitions=[{'AttributeName':'forum_name','AttributeType':'S'}],
                      ProvisionedThroughput={'ReadCapacityUnits':5,'WriteCapacityUnits':5})

    with assert_raises(ClientError) as ex:
        conn.put_item(
            TableName=name,
            Item={
                'forum_name': { 'S': 'LOLCat Forum' },
                'subject': { 'S': 'Check this out!' },
                'Body': { 'S': 'http://url_to_lolcat.gif'},
                'SentBy': { 'S': "" },
                'ReceivedTime': { 'S': '12/9/2011 11:36:03 PM'},
            }
        )

    ex.exception.response['Error']['Code'].should.equal('ValidationException')
    ex.exception.response['ResponseMetadata']['HTTPStatusCode'].should.equal(400)
    ex.exception.response['Error']['Message'].should.equal(
        'One or more parameter values were invalid: An AttributeValue may not contain an empty string'
    )


@requires_boto_gte("2.9")
@mock_dynamodb2
def test_query_invalid_table():
    conn = boto3.client('dynamodb',
                        region_name='us-west-2',
                        aws_access_key_id="ak",
                        aws_secret_access_key="sk")
    try:
        conn.query(TableName='invalid_table', KeyConditionExpression='index1 = :partitionkeyval', ExpressionAttributeValues={':partitionkeyval': {'S':'test'}})
    except ClientError as exception:
        assert exception.response['Error']['Code'] == "ResourceNotFoundException"


@requires_boto_gte("2.9")
@mock_dynamodb2
def test_scan_returns_consumed_capacity():
    name = 'TestTable'
    conn = boto3.client('dynamodb',
                        region_name='us-west-2',
                        aws_access_key_id="ak",
                        aws_secret_access_key="sk")

    conn.create_table(TableName=name,
                      KeySchema=[{'AttributeName':'forum_name','KeyType':'HASH'}],
                      AttributeDefinitions=[{'AttributeName':'forum_name','AttributeType':'S'}],
                      ProvisionedThroughput={'ReadCapacityUnits':5,'WriteCapacityUnits':5})

    conn.put_item(
            TableName=name,
            Item={
                'forum_name': { 'S': 'LOLCat Forum' },
                'subject': { 'S': 'Check this out!' },
                'Body': { 'S': 'http://url_to_lolcat.gif'},
                'SentBy': { 'S': "test" },
                'ReceivedTime': { 'S': '12/9/2011 11:36:03 PM'},
            }
        )

    response = conn.scan(
        TableName=name,
    )

    assert 'ConsumedCapacity' in response
    assert 'CapacityUnits' in response['ConsumedCapacity']
    assert response['ConsumedCapacity']['TableName'] == name

@requires_boto_gte("2.9")
@mock_dynamodb2
def test_query_returns_consumed_capacity():
    dynamodb = boto3.resource('dynamodb', region_name='us-east-1')

    # Create the DynamoDB table.
    table = dynamodb.create_table(
        TableName='users',
        KeySchema=[
            {
                'AttributeName': 'forum_name',
                'KeyType': 'HASH'
            },
            {
                'AttributeName': 'subject',
                'KeyType': 'RANGE'
            },
        ],
        AttributeDefinitions=[
            {
                'AttributeName': 'forum_name',
                'AttributeType': 'S'
            },
            {
                'AttributeName': 'subject',
                'AttributeType': 'S'
            },
        ],
        ProvisionedThroughput={
            'ReadCapacityUnits': 5,
            'WriteCapacityUnits': 5
        }
    )
    table = dynamodb.Table('users')

    table.put_item(Item={
        'forum_name': 'the-key',
        'subject': '123',
        'body': 'some test message'
    })

    results = table.query(
        KeyConditionExpression=Key('forum_name').eq(
            'the-key')
    )

    assert 'ConsumedCapacity' in results
    assert 'CapacityUnits' in results['ConsumedCapacity']
    assert results['ConsumedCapacity']['CapacityUnits'] == 1

@mock_dynamodb2
def test_basic_projection_expressions():
    dynamodb = boto3.resource('dynamodb', region_name='us-east-1')

    # Create the DynamoDB table.
    table = dynamodb.create_table(
        TableName='users',
        KeySchema=[
            {
                'AttributeName': 'forum_name',
                'KeyType': 'HASH'
            },
            {
                'AttributeName': 'subject',
                'KeyType': 'RANGE'
            },
        ],
        AttributeDefinitions=[
            {
                'AttributeName': 'forum_name',
                'AttributeType': 'S'
            },
            {
                'AttributeName': 'subject',
                'AttributeType': 'S'
            },
        ],
        ProvisionedThroughput={
            'ReadCapacityUnits': 5,
            'WriteCapacityUnits': 5
        }
    )
    table = dynamodb.Table('users')

    table.put_item(Item={
        'forum_name': 'the-key',
        'subject': '123',
        'body': 'some test message'
    })

    table.put_item(Item={
        'forum_name': 'not-the-key',
        'subject': '123',
        'body': 'some other test message'
    })
    # Test a query returning all items
    results = table.query(
        KeyConditionExpression=Key('forum_name').eq(
            'the-key'),
        ProjectionExpression='body, subject'
    )

    assert 'body' in results['Items'][0]
    assert results['Items'][0]['body'] == 'some test message'
    assert 'subject' in results['Items'][0]

    table.put_item(Item={
        'forum_name': 'the-key',
        'subject': '1234',
        'body': 'yet another test message'
    })

    results = table.query(
        KeyConditionExpression=Key('forum_name').eq(
            'the-key'),
        ProjectionExpression='body'
    )

    assert 'body' in results['Items'][0]
    assert results['Items'][0]['body'] == 'some test message'
    assert 'body' in results['Items'][1]
    assert results['Items'][1]['body'] == 'yet another test message'

@mock_dynamodb2
def test_basic_projection_expressions_with_attr_expression_names():
    dynamodb = boto3.resource('dynamodb', region_name='us-east-1')

    # Create the DynamoDB table.
    table = dynamodb.create_table(
        TableName='users',
        KeySchema=[
            {
                'AttributeName': 'forum_name',
                'KeyType': 'HASH'
            },
            {
                'AttributeName': 'subject',
                'KeyType': 'RANGE'
            },
        ],
        AttributeDefinitions=[
            {
                'AttributeName': 'forum_name',
                'AttributeType': 'S'
            },
            {
                'AttributeName': 'subject',
                'AttributeType': 'S'
            },
        ],
        ProvisionedThroughput={
            'ReadCapacityUnits': 5,
            'WriteCapacityUnits': 5
        }
    )
    table = dynamodb.Table('users')

    table.put_item(Item={
        'forum_name': 'the-key',
        'subject': '123',
        'body': 'some test message',
        'attachment': 'something'
    })

    table.put_item(Item={
        'forum_name': 'not-the-key',
        'subject': '123',
        'body': 'some other test message',
        'attachment': 'something'
    })
    # Test a query returning all items

    results = table.query(
        KeyConditionExpression=Key('forum_name').eq(
            'the-key'),
        ProjectionExpression='#rl, #rt, subject',
        ExpressionAttributeNames={
            '#rl': 'body',
            '#rt': 'attachment'
            },
    )

    assert 'body' in results['Items'][0]
    assert results['Items'][0]['body'] == 'some test message'
    assert 'subject' in results['Items'][0]
    assert results['Items'][0]['subject'] == '123'
    assert 'attachment' in results['Items'][0]
    assert results['Items'][0]['attachment'] == 'something'

@mock_dynamodb2
def test_put_item_returns_consumed_capacity():
    dynamodb = boto3.resource('dynamodb', region_name='us-east-1')

    # Create the DynamoDB table.
    table = dynamodb.create_table(
        TableName='users',
        KeySchema=[
            {
                'AttributeName': 'forum_name',
                'KeyType': 'HASH'
            },
            {
                'AttributeName': 'subject',
                'KeyType': 'RANGE'
            },
        ],
        AttributeDefinitions=[
            {
                'AttributeName': 'forum_name',
                'AttributeType': 'S'
            },
            {
                'AttributeName': 'subject',
                'AttributeType': 'S'
            },
        ],
        ProvisionedThroughput={
            'ReadCapacityUnits': 5,
            'WriteCapacityUnits': 5
        }
    )
    table = dynamodb.Table('users')

    response = table.put_item(Item={
        'forum_name': 'the-key',
        'subject': '123',
        'body': 'some test message',
    })

    assert 'ConsumedCapacity' in response

@mock_dynamodb2
def test_update_item_returns_consumed_capacity():
    dynamodb = boto3.resource('dynamodb', region_name='us-east-1')

    # Create the DynamoDB table.
    table = dynamodb.create_table(
        TableName='users',
        KeySchema=[
            {
                'AttributeName': 'forum_name',
                'KeyType': 'HASH'
            },
            {
                'AttributeName': 'subject',
                'KeyType': 'RANGE'
            },
        ],
        AttributeDefinitions=[
            {
                'AttributeName': 'forum_name',
                'AttributeType': 'S'
            },
            {
                'AttributeName': 'subject',
                'AttributeType': 'S'
            },
        ],
        ProvisionedThroughput={
            'ReadCapacityUnits': 5,
            'WriteCapacityUnits': 5
        }
    )
    table = dynamodb.Table('users')

    table.put_item(Item={
        'forum_name': 'the-key',
        'subject': '123',
        'body': 'some test message',
    })

    response = table.update_item(Key={
        'forum_name': 'the-key',
        'subject': '123'
        },
        UpdateExpression='set body=:tb',
        ExpressionAttributeValues={
            ':tb': 'a new message'
    })

    assert 'ConsumedCapacity' in response
    assert 'CapacityUnits' in response['ConsumedCapacity']
    assert 'TableName' in response['ConsumedCapacity']

@mock_dynamodb2
def test_get_item_returns_consumed_capacity():
    dynamodb = boto3.resource('dynamodb', region_name='us-east-1')

    # Create the DynamoDB table.
    table = dynamodb.create_table(
        TableName='users',
        KeySchema=[
            {
                'AttributeName': 'forum_name',
                'KeyType': 'HASH'
            },
            {
                'AttributeName': 'subject',
                'KeyType': 'RANGE'
            },
        ],
        AttributeDefinitions=[
            {
                'AttributeName': 'forum_name',
                'AttributeType': 'S'
            },
            {
                'AttributeName': 'subject',
                'AttributeType': 'S'
            },
        ],
        ProvisionedThroughput={
            'ReadCapacityUnits': 5,
            'WriteCapacityUnits': 5
        }
    )
    table = dynamodb.Table('users')

    table.put_item(Item={
        'forum_name': 'the-key',
        'subject': '123',
        'body': 'some test message',
    })

    response = table.get_item(Key={
        'forum_name': 'the-key',
        'subject': '123'
    })

    assert 'ConsumedCapacity' in response
    assert 'CapacityUnits' in response['ConsumedCapacity']
    assert 'TableName' in response['ConsumedCapacity']



from __future__ import unicode_literals

import sure  # noqa
from freezegun import freeze_time

from moto.core.utils import camelcase_to_underscores, underscores_to_camelcase, unix_time


def test_camelcase_to_underscores():
    cases = {
        "theNewAttribute": "the_new_attribute",
        "attri bute With Space": "attribute_with_space",
        "FirstLetterCapital": "first_letter_capital",
        "ListMFADevices": "list_mfa_devices",
    }
    for arg, expected in cases.items():
        camelcase_to_underscores(arg).should.equal(expected)


def test_underscores_to_camelcase():
    cases = {
        "the_new_attribute": "theNewAttribute",
    }
    for arg, expected in cases.items():
        underscores_to_camelcase(arg).should.equal(expected)


@freeze_time("2015-01-01 12:00:00")
def test_unix_time():
    unix_time().should.equal(1420113600.0)



from __future__ import unicode_literals
# Ensure 'assert_raises' context manager support for Python 2.6
import tests.backport_assert_raises
from nose.tools import assert_raises

import boto
from boto.exception import EC2ResponseError
import sure  # noqa

from moto import mock_ec2_deprecated
from tests.helpers import requires_boto_gte


@requires_boto_gte("2.32.0")
@mock_ec2_deprecated
def test_vpc_peering_connections():
    conn = boto.connect_vpc('the_key', 'the_secret')
    vpc = conn.create_vpc("10.0.0.0/16")
    peer_vpc = conn.create_vpc("11.0.0.0/16")

    vpc_pcx = conn.create_vpc_peering_connection(vpc.id, peer_vpc.id)
    vpc_pcx._status.code.should.equal('initiating-request')

    return vpc_pcx


@requires_boto_gte("2.32.0")
@mock_ec2_deprecated
def test_vpc_peering_connections_get_all():
    conn = boto.connect_vpc('the_key', 'the_secret')
    vpc_pcx = test_vpc_peering_connections()
    vpc_pcx._status.code.should.equal('initiating-request')

    all_vpc_pcxs = conn.get_all_vpc_peering_connections()
    all_vpc_pcxs.should.have.length_of(1)
    all_vpc_pcxs[0]._status.code.should.equal('pending-acceptance')


@requires_boto_gte("2.32.0")
@mock_ec2_deprecated
def test_vpc_peering_connections_accept():
    conn = boto.connect_vpc('the_key', 'the_secret')
    vpc_pcx = test_vpc_peering_connections()

    vpc_pcx = conn.accept_vpc_peering_connection(vpc_pcx.id)
    vpc_pcx._status.code.should.equal('active')

    with assert_raises(EC2ResponseError) as cm:
        conn.reject_vpc_peering_connection(vpc_pcx.id)
    cm.exception.code.should.equal('InvalidStateTransition')
    cm.exception.status.should.equal(400)
    cm.exception.request_id.should_not.be.none

    all_vpc_pcxs = conn.get_all_vpc_peering_connections()
    all_vpc_pcxs.should.have.length_of(1)
    all_vpc_pcxs[0]._status.code.should.equal('active')


@requires_boto_gte("2.32.0")
@mock_ec2_deprecated
def test_vpc_peering_connections_reject():
    conn = boto.connect_vpc('the_key', 'the_secret')
    vpc_pcx = test_vpc_peering_connections()

    verdict = conn.reject_vpc_peering_connection(vpc_pcx.id)
    verdict.should.equal(True)

    with assert_raises(EC2ResponseError) as cm:
        conn.accept_vpc_peering_connection(vpc_pcx.id)
    cm.exception.code.should.equal('InvalidStateTransition')
    cm.exception.status.should.equal(400)
    cm.exception.request_id.should_not.be.none

    all_vpc_pcxs = conn.get_all_vpc_peering_connections()
    all_vpc_pcxs.should.have.length_of(1)
    all_vpc_pcxs[0]._status.code.should.equal('rejected')


@requires_boto_gte("2.32.1")
@mock_ec2_deprecated
def test_vpc_peering_connections_delete():
    conn = boto.connect_vpc('the_key', 'the_secret')
    vpc_pcx = test_vpc_peering_connections()

    verdict = vpc_pcx.delete()
    verdict.should.equal(True)

    all_vpc_pcxs = conn.get_all_vpc_peering_connections()
    all_vpc_pcxs.should.have.length_of(0)

    with assert_raises(EC2ResponseError) as cm:
        conn.delete_vpc_peering_connection("pcx-1234abcd")
    cm.exception.code.should.equal('InvalidVpcPeeringConnectionId.NotFound')
    cm.exception.status.should.equal(400)
    cm.exception.request_id.should_not.be.none



from __future__ import unicode_literals
from .responses import LambdaResponse

url_bases = [
    "https?://lambda.(.+).amazonaws.com",
]

response = LambdaResponse()

url_paths = {
    '{0}/(?P<api_version>[^/]+)/functions/?$': response.root,
    r'{0}/(?P<api_version>[^/]+)/functions/(?P<function_name>[\w_-]+)/?$': response.function,
    r'{0}/(?P<api_version>[^/]+)/functions/(?P<function_name>[\w_-]+)/invocations/?$': response.invoke,
    r'{0}/(?P<api_version>[^/]+)/functions/(?P<function_name>[\w_-]+)/invoke-async/?$': response.invoke_async,
    r'{0}/(?P<api_version>[^/]+)/tags/(?P<resource_arn>.+)': response.tag
}



from __future__ import unicode_literals
import re

import boto3
import boto.kms
from boto.exception import JSONResponseError
from boto.kms.exceptions import AlreadyExistsException, NotFoundException
import sure  # noqa
from moto import mock_kms, mock_kms_deprecated
from nose.tools import assert_raises


@mock_kms_deprecated
def test_create_key():
    conn = boto.kms.connect_to_region("us-west-2")

    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')

    key['KeyMetadata']['Description'].should.equal("my key")
    key['KeyMetadata']['KeyUsage'].should.equal("ENCRYPT_DECRYPT")
    key['KeyMetadata']['Enabled'].should.equal(True)


@mock_kms_deprecated
def test_describe_key():
    conn = boto.kms.connect_to_region("us-west-2")
    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    key = conn.describe_key(key_id)
    key['KeyMetadata']['Description'].should.equal("my key")
    key['KeyMetadata']['KeyUsage'].should.equal("ENCRYPT_DECRYPT")


@mock_kms_deprecated
def test_describe_key_via_alias():
    conn = boto.kms.connect_to_region("us-west-2")
    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    conn.create_alias(alias_name='alias/my-key-alias',
                      target_key_id=key['KeyMetadata']['KeyId'])

    alias_key = conn.describe_key('alias/my-key-alias')
    alias_key['KeyMetadata']['Description'].should.equal("my key")
    alias_key['KeyMetadata']['KeyUsage'].should.equal("ENCRYPT_DECRYPT")
    alias_key['KeyMetadata']['Arn'].should.equal(key['KeyMetadata']['Arn'])


@mock_kms_deprecated
def test_describe_key_via_alias_not_found():
    conn = boto.kms.connect_to_region("us-west-2")
    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    conn.create_alias(alias_name='alias/my-key-alias',
                      target_key_id=key['KeyMetadata']['KeyId'])

    conn.describe_key.when.called_with(
        'alias/not-found-alias').should.throw(JSONResponseError)


@mock_kms_deprecated
def test_describe_key_via_arn():
    conn = boto.kms.connect_to_region("us-west-2")
    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    arn = key['KeyMetadata']['Arn']

    the_key = conn.describe_key(arn)
    the_key['KeyMetadata']['Description'].should.equal("my key")
    the_key['KeyMetadata']['KeyUsage'].should.equal("ENCRYPT_DECRYPT")
    the_key['KeyMetadata']['KeyId'].should.equal(key['KeyMetadata']['KeyId'])


@mock_kms_deprecated
def test_describe_missing_key():
    conn = boto.kms.connect_to_region("us-west-2")
    conn.describe_key.when.called_with(
        "not-a-key").should.throw(JSONResponseError)


@mock_kms_deprecated
def test_list_keys():
    conn = boto.kms.connect_to_region("us-west-2")

    conn.create_key(policy="my policy", description="my key1",
                    key_usage='ENCRYPT_DECRYPT')
    conn.create_key(policy="my policy", description="my key2",
                    key_usage='ENCRYPT_DECRYPT')

    keys = conn.list_keys()
    keys['Keys'].should.have.length_of(2)


@mock_kms_deprecated
def test_enable_key_rotation():
    conn = boto.kms.connect_to_region("us-west-2")

    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    conn.enable_key_rotation(key_id)

    conn.get_key_rotation_status(
        key_id)['KeyRotationEnabled'].should.equal(True)


@mock_kms_deprecated
def test_enable_key_rotation_via_arn():
    conn = boto.kms.connect_to_region("us-west-2")

    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['Arn']

    conn.enable_key_rotation(key_id)

    conn.get_key_rotation_status(
        key_id)['KeyRotationEnabled'].should.equal(True)


@mock_kms_deprecated
def test_enable_key_rotation_with_missing_key():
    conn = boto.kms.connect_to_region("us-west-2")
    conn.enable_key_rotation.when.called_with(
        "not-a-key").should.throw(JSONResponseError)


@mock_kms_deprecated
def test_enable_key_rotation_with_alias_name_should_fail():
    conn = boto.kms.connect_to_region("us-west-2")
    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    conn.create_alias(alias_name='alias/my-key-alias',
                      target_key_id=key['KeyMetadata']['KeyId'])

    alias_key = conn.describe_key('alias/my-key-alias')
    alias_key['KeyMetadata']['Arn'].should.equal(key['KeyMetadata']['Arn'])

    conn.enable_key_rotation.when.called_with(
        'alias/my-alias').should.throw(JSONResponseError)


@mock_kms_deprecated
def test_disable_key_rotation():
    conn = boto.kms.connect_to_region("us-west-2")

    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    conn.enable_key_rotation(key_id)
    conn.get_key_rotation_status(
        key_id)['KeyRotationEnabled'].should.equal(True)

    conn.disable_key_rotation(key_id)
    conn.get_key_rotation_status(
        key_id)['KeyRotationEnabled'].should.equal(False)


@mock_kms_deprecated
def test_encrypt():
    """
    test_encrypt
    Using base64 encoding to merely test that the endpoint was called
    """
    conn = boto.kms.connect_to_region("us-west-2")
    response = conn.encrypt('key_id', 'encryptme'.encode('utf-8'))
    response['CiphertextBlob'].should.equal(b'ZW5jcnlwdG1l')


@mock_kms_deprecated
def test_decrypt():
    conn = boto.kms.connect_to_region('us-west-2')
    response = conn.decrypt('ZW5jcnlwdG1l'.encode('utf-8'))
    response['Plaintext'].should.equal(b'encryptme')


@mock_kms_deprecated
def test_disable_key_rotation_with_missing_key():
    conn = boto.kms.connect_to_region("us-west-2")
    conn.disable_key_rotation.when.called_with(
        "not-a-key").should.throw(JSONResponseError)


@mock_kms_deprecated
def test_get_key_rotation_status_with_missing_key():
    conn = boto.kms.connect_to_region("us-west-2")
    conn.get_key_rotation_status.when.called_with(
        "not-a-key").should.throw(JSONResponseError)


@mock_kms_deprecated
def test_get_key_rotation_status():
    conn = boto.kms.connect_to_region("us-west-2")

    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    conn.get_key_rotation_status(
        key_id)['KeyRotationEnabled'].should.equal(False)


@mock_kms_deprecated
def test_create_key_defaults_key_rotation():
    conn = boto.kms.connect_to_region("us-west-2")

    key = conn.create_key(policy="my policy",
                          description="my key", key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    conn.get_key_rotation_status(
        key_id)['KeyRotationEnabled'].should.equal(False)


@mock_kms_deprecated
def test_get_key_policy():
    conn = boto.kms.connect_to_region('us-west-2')

    key = conn.create_key(policy='my policy',
                          description='my key1', key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    policy = conn.get_key_policy(key_id, 'default')
    policy['Policy'].should.equal('my policy')


@mock_kms_deprecated
def test_get_key_policy_via_arn():
    conn = boto.kms.connect_to_region('us-west-2')

    key = conn.create_key(policy='my policy',
                          description='my key1', key_usage='ENCRYPT_DECRYPT')
    policy = conn.get_key_policy(key['KeyMetadata']['Arn'], 'default')

    policy['Policy'].should.equal('my policy')


@mock_kms_deprecated
def test_put_key_policy():
    conn = boto.kms.connect_to_region('us-west-2')

    key = conn.create_key(policy='my policy',
                          description='my key1', key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    conn.put_key_policy(key_id, 'default', 'new policy')
    policy = conn.get_key_policy(key_id, 'default')
    policy['Policy'].should.equal('new policy')


@mock_kms_deprecated
def test_put_key_policy_via_arn():
    conn = boto.kms.connect_to_region('us-west-2')

    key = conn.create_key(policy='my policy',
                          description='my key1', key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['Arn']

    conn.put_key_policy(key_id, 'default', 'new policy')
    policy = conn.get_key_policy(key_id, 'default')
    policy['Policy'].should.equal('new policy')


@mock_kms_deprecated
def test_put_key_policy_via_alias_should_not_update():
    conn = boto.kms.connect_to_region('us-west-2')

    key = conn.create_key(policy='my policy',
                          description='my key1', key_usage='ENCRYPT_DECRYPT')
    conn.create_alias(alias_name='alias/my-key-alias',
                      target_key_id=key['KeyMetadata']['KeyId'])

    conn.put_key_policy.when.called_with(
        'alias/my-key-alias', 'default', 'new policy').should.throw(JSONResponseError)

    policy = conn.get_key_policy(key['KeyMetadata']['KeyId'], 'default')
    policy['Policy'].should.equal('my policy')


@mock_kms_deprecated
def test_put_key_policy():
    conn = boto.kms.connect_to_region('us-west-2')

    key = conn.create_key(policy='my policy',
                          description='my key1', key_usage='ENCRYPT_DECRYPT')
    conn.put_key_policy(key['KeyMetadata']['Arn'], 'default', 'new policy')

    policy = conn.get_key_policy(key['KeyMetadata']['KeyId'], 'default')
    policy['Policy'].should.equal('new policy')


@mock_kms_deprecated
def test_list_key_policies():
    conn = boto.kms.connect_to_region('us-west-2')

    key = conn.create_key(policy='my policy',
                          description='my key1', key_usage='ENCRYPT_DECRYPT')
    key_id = key['KeyMetadata']['KeyId']

    policies = conn.list_key_policies(key_id)
    policies['PolicyNames'].should.equal(['default'])


@mock_kms_deprecated
def test__create_alias__returns_none_if_correct():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']

    resp = kms.create_alias('alias/my-alias', key_id)

    resp.should.be.none


@mock_kms_deprecated
def test__create_alias__raises_if_reserved_alias():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']

    reserved_aliases = [
        'alias/aws/ebs',
        'alias/aws/s3',
        'alias/aws/redshift',
        'alias/aws/rds',
    ]

    for alias_name in reserved_aliases:
        with assert_raises(JSONResponseError) as err:
            kms.create_alias(alias_name, key_id)

        ex = err.exception
        ex.error_message.should.be.none
        ex.error_code.should.equal('NotAuthorizedException')
        ex.body.should.equal({'__type': 'NotAuthorizedException'})
        ex.reason.should.equal('Bad Request')
        ex.status.should.equal(400)


@mock_kms_deprecated
def test__create_alias__can_create_multiple_aliases_for_same_key_id():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']

    kms.create_alias('alias/my-alias3', key_id).should.be.none
    kms.create_alias('alias/my-alias4', key_id).should.be.none
    kms.create_alias('alias/my-alias5', key_id).should.be.none


@mock_kms_deprecated
def test__create_alias__raises_if_wrong_prefix():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']

    with assert_raises(JSONResponseError) as err:
        kms.create_alias('wrongprefix/my-alias', key_id)

    ex = err.exception
    ex.error_message.should.equal('Invalid identifier')
    ex.error_code.should.equal('ValidationException')
    ex.body.should.equal({'message': 'Invalid identifier',
                          '__type': 'ValidationException'})
    ex.reason.should.equal('Bad Request')
    ex.status.should.equal(400)


@mock_kms_deprecated
def test__create_alias__raises_if_duplicate():
    region = 'us-west-2'
    kms = boto.kms.connect_to_region(region)
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']
    alias = 'alias/my-alias'

    kms.create_alias(alias, key_id)

    with assert_raises(AlreadyExistsException) as err:
        kms.create_alias(alias, key_id)

    ex = err.exception
    ex.error_message.should.match(r'An alias with the name arn:aws:kms:{region}:\d{{12}}:{alias} already exists'
                                  .format(**locals()))
    ex.error_code.should.be.none
    ex.box_usage.should.be.none
    ex.request_id.should.be.none
    ex.body['message'].should.match(r'An alias with the name arn:aws:kms:{region}:\d{{12}}:{alias} already exists'
                                    .format(**locals()))
    ex.body['__type'].should.equal('AlreadyExistsException')
    ex.reason.should.equal('Bad Request')
    ex.status.should.equal(400)


@mock_kms_deprecated
def test__create_alias__raises_if_alias_has_restricted_characters():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']

    alias_names_with_restricted_characters = [
        'alias/my-alias!',
        'alias/my-alias$',
        'alias/my-alias@',
    ]

    for alias_name in alias_names_with_restricted_characters:
        with assert_raises(JSONResponseError) as err:
            kms.create_alias(alias_name, key_id)
        ex = err.exception
        ex.body['__type'].should.equal('ValidationException')
        ex.body['message'].should.equal(
            "1 validation error detected: Value '{alias_name}' at 'aliasName' failed to satisfy constraint: Member must satisfy regular expression pattern: ^[a-zA-Z0-9:/_-]+$".format(**locals()))
        ex.error_code.should.equal('ValidationException')
        ex.message.should.equal(
            "1 validation error detected: Value '{alias_name}' at 'aliasName' failed to satisfy constraint: Member must satisfy regular expression pattern: ^[a-zA-Z0-9:/_-]+$".format(**locals()))
        ex.reason.should.equal('Bad Request')
        ex.status.should.equal(400)


@mock_kms_deprecated
def test__create_alias__raises_if_alias_has_colon_character():
    # For some reason, colons are not accepted for an alias, even though they
    # are accepted by regex ^[a-zA-Z0-9:/_-]+$
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']

    alias_names_with_restricted_characters = [
        'alias/my:alias',
    ]

    for alias_name in alias_names_with_restricted_characters:
        with assert_raises(JSONResponseError) as err:
            kms.create_alias(alias_name, key_id)
        ex = err.exception
        ex.body['__type'].should.equal('ValidationException')
        ex.body['message'].should.equal(
            "{alias_name} contains invalid characters for an alias".format(**locals()))
        ex.error_code.should.equal('ValidationException')
        ex.message.should.equal(
            "{alias_name} contains invalid characters for an alias".format(**locals()))
        ex.reason.should.equal('Bad Request')
        ex.status.should.equal(400)


@mock_kms_deprecated
def test__create_alias__accepted_characters():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']

    alias_names_with_accepted_characters = [
        'alias/my-alias_/',
        'alias/my_alias-/',
    ]

    for alias_name in alias_names_with_accepted_characters:
        kms.create_alias(alias_name, key_id)


@mock_kms_deprecated
def test__create_alias__raises_if_target_key_id_is_existing_alias():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']
    alias = 'alias/my-alias'

    kms.create_alias(alias, key_id)

    with assert_raises(JSONResponseError) as err:
        kms.create_alias(alias, alias)

    ex = err.exception
    ex.body['__type'].should.equal('ValidationException')
    ex.body['message'].should.equal('Aliases must refer to keys. Not aliases')
    ex.error_code.should.equal('ValidationException')
    ex.message.should.equal('Aliases must refer to keys. Not aliases')
    ex.reason.should.equal('Bad Request')
    ex.status.should.equal(400)


@mock_kms_deprecated
def test__delete_alias():
    kms = boto.connect_kms()
    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']
    alias = 'alias/my-alias'

    kms.create_alias(alias, key_id)

    resp = kms.delete_alias(alias)

    resp.should.be.none

    # we can create the alias again, since it has been deleted
    kms.create_alias(alias, key_id)


@mock_kms_deprecated
def test__delete_alias__raises_if_wrong_prefix():
    kms = boto.connect_kms()

    with assert_raises(JSONResponseError) as err:
        kms.delete_alias('wrongprefix/my-alias')

    ex = err.exception
    ex.body['__type'].should.equal('ValidationException')
    ex.body['message'].should.equal('Invalid identifier')
    ex.error_code.should.equal('ValidationException')
    ex.message.should.equal('Invalid identifier')
    ex.reason.should.equal('Bad Request')
    ex.status.should.equal(400)


@mock_kms_deprecated
def test__delete_alias__raises_if_alias_is_not_found():
    region = 'us-west-2'
    kms = boto.kms.connect_to_region(region)
    alias_name = 'alias/unexisting-alias'

    with assert_raises(NotFoundException) as err:
        kms.delete_alias(alias_name)

    ex = err.exception
    ex.body['__type'].should.equal('NotFoundException')
    ex.body['message'].should.match(
        r'Alias arn:aws:kms:{region}:\d{{12}}:{alias_name} is not found.'.format(**locals()))
    ex.box_usage.should.be.none
    ex.error_code.should.be.none
    ex.message.should.match(
        r'Alias arn:aws:kms:{region}:\d{{12}}:{alias_name} is not found.'.format(**locals()))
    ex.reason.should.equal('Bad Request')
    ex.request_id.should.be.none
    ex.status.should.equal(400)


@mock_kms_deprecated
def test__list_aliases():
    region = "eu-west-1"
    kms = boto.kms.connect_to_region(region)

    create_resp = kms.create_key()
    key_id = create_resp['KeyMetadata']['KeyId']
    kms.create_alias('alias/my-alias1', key_id)
    kms.create_alias('alias/my-alias2', key_id)
    kms.create_alias('alias/my-alias3', key_id)

    resp = kms.list_aliases()

    resp['Truncated'].should.be.false

    aliases = resp['Aliases']

    def has_correct_arn(alias_obj):
        alias_name = alias_obj['AliasName']
        alias_arn = alias_obj['AliasArn']
        return re.match(r'arn:aws:kms:{region}:\d{{12}}:{alias_name}'.format(region=region, alias_name=alias_name),
                        alias_arn)

    len([alias for alias in aliases if
         has_correct_arn(alias) and 'alias/aws/ebs' == alias['AliasName']]).should.equal(1)
    len([alias for alias in aliases if
         has_correct_arn(alias) and 'alias/aws/rds' == alias['AliasName']]).should.equal(1)
    len([alias for alias in aliases if
         has_correct_arn(alias) and 'alias/aws/redshift' == alias['AliasName']]).should.equal(1)
    len([alias for alias in aliases if
         has_correct_arn(alias) and 'alias/aws/s3' == alias['AliasName']]).should.equal(1)

    len([alias for alias in aliases if
         has_correct_arn(alias) and 'alias/my-alias1' == alias['AliasName']]).should.equal(1)
    len([alias for alias in aliases if
         has_correct_arn(alias) and 'alias/my-alias2' == alias['AliasName']]).should.equal(1)

    len([alias for alias in aliases if 'TargetKeyId' in alias and key_id ==
         alias['TargetKeyId']]).should.equal(3)

    len(aliases).should.equal(7)


@mock_kms_deprecated
def test__assert_valid_key_id():
    from moto.kms.responses import _assert_valid_key_id
    import uuid

    _assert_valid_key_id.when.called_with(
        "not-a-key").should.throw(JSONResponseError)
    _assert_valid_key_id.when.called_with(
        str(uuid.uuid4())).should_not.throw(JSONResponseError)


@mock_kms_deprecated
def test__assert_default_policy():
    from moto.kms.responses import _assert_default_policy

    _assert_default_policy.when.called_with(
        "not-default").should.throw(JSONResponseError)
    _assert_default_policy.when.called_with(
        "default").should_not.throw(JSONResponseError)


@mock_kms
def test_kms_encrypt_boto3():
    client = boto3.client('kms', region_name='us-east-1')
    response = client.encrypt(KeyId='foo', Plaintext=b'bar')

    response = client.decrypt(CiphertextBlob=response['CiphertextBlob'])
    response['Plaintext'].should.equal(b'bar')



from __future__ import unicode_literals
"""
Patch courtesy of:
https://marmida.com/blog/index.php/2012/08/08/monkey-patching-assert_raises/
"""

# code for monkey-patching
import nose.tools

# let's fix nose.tools.assert_raises (which is really unittest.assertRaises)
# so that it always supports context management

# in order for these changes to be available to other modules, you'll need
# to guarantee this module is imported by your fixture before either nose or
# unittest are imported

try:
    nose.tools.assert_raises(Exception)
except TypeError:
    # this version of assert_raises doesn't support the 1-arg version
    class AssertRaisesContext(object):

        def __init__(self, expected):
            self.expected = expected

        def __enter__(self):
            return self

        def __exit__(self, exc_type, exc_val, tb):
            self.exception = exc_val
            if issubclass(exc_type, self.expected):
                return True
            nose.tools.assert_equal(exc_type, self.expected)
            # if you get to this line, the last assertion must have passed
            # suppress the propagation of this exception
            return True

    def assert_raises_context(exc_type):
        return AssertRaisesContext(exc_type)

    nose.tools.assert_raises = assert_raises_context



from __future__ import unicode_literals
from .models import opsworks_backends
from ..core.models import base_decorator, deprecated_base_decorator

opsworks_backend = opsworks_backends['us-east-1']
mock_opsworks = base_decorator(opsworks_backends)
mock_opsworks_deprecated = deprecated_base_decorator(opsworks_backends)



from boto.swf.exceptions import SWFResponseError
from freezegun import freeze_time

from moto import mock_swf_deprecated
from moto.swf import swf_backend

from ..utils import setup_workflow, SCHEDULE_ACTIVITY_TASK_DECISION


# PollForActivityTask endpoint
@mock_swf_deprecated
def test_poll_for_activity_task_when_one():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    resp = conn.poll_for_activity_task(
        "test-domain", "activity-task-list", identity="surprise")
    resp["activityId"].should.equal("my-activity-001")
    resp["taskToken"].should_not.be.none

    resp = conn.get_workflow_execution_history(
        "test-domain", conn.run_id, "uid-abcd1234")
    resp["events"][-1]["eventType"].should.equal("ActivityTaskStarted")
    resp["events"][-1]["activityTaskStartedEventAttributes"].should.equal(
        {"identity": "surprise", "scheduledEventId": 5}
    )


@mock_swf_deprecated
def test_poll_for_activity_task_when_none():
    conn = setup_workflow()
    resp = conn.poll_for_activity_task("test-domain", "activity-task-list")
    resp.should.equal({"startedEventId": 0})


@mock_swf_deprecated
def test_poll_for_activity_task_on_non_existent_queue():
    conn = setup_workflow()
    resp = conn.poll_for_activity_task("test-domain", "non-existent-queue")
    resp.should.equal({"startedEventId": 0})


# CountPendingActivityTasks endpoint
@mock_swf_deprecated
def test_count_pending_activity_tasks():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])

    resp = conn.count_pending_activity_tasks(
        "test-domain", "activity-task-list")
    resp.should.equal({"count": 1, "truncated": False})


@mock_swf_deprecated
def test_count_pending_decision_tasks_on_non_existent_task_list():
    conn = setup_workflow()
    resp = conn.count_pending_activity_tasks("test-domain", "non-existent")
    resp.should.equal({"count": 0, "truncated": False})


# RespondActivityTaskCompleted endpoint
@mock_swf_deprecated
def test_respond_activity_task_completed():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    activity_token = conn.poll_for_activity_task(
        "test-domain", "activity-task-list")["taskToken"]

    resp = conn.respond_activity_task_completed(
        activity_token, result="result of the task")
    resp.should.be.none

    resp = conn.get_workflow_execution_history(
        "test-domain", conn.run_id, "uid-abcd1234")
    resp["events"][-2]["eventType"].should.equal("ActivityTaskCompleted")
    resp["events"][-2]["activityTaskCompletedEventAttributes"].should.equal(
        {"result": "result of the task", "scheduledEventId": 5, "startedEventId": 6}
    )


@mock_swf_deprecated
def test_respond_activity_task_completed_on_closed_workflow_execution():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    activity_token = conn.poll_for_activity_task(
        "test-domain", "activity-task-list")["taskToken"]

    # bad: we're closing workflow execution manually, but endpoints are not
    # coded for now..
    wfe = swf_backend.domains[0].workflow_executions[-1]
    wfe.execution_status = "CLOSED"
    # /bad

    conn.respond_activity_task_completed.when.called_with(
        activity_token
    ).should.throw(SWFResponseError, "WorkflowExecution=")


@mock_swf_deprecated
def test_respond_activity_task_completed_with_task_already_completed():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    activity_token = conn.poll_for_activity_task(
        "test-domain", "activity-task-list")["taskToken"]

    conn.respond_activity_task_completed(activity_token)

    conn.respond_activity_task_completed.when.called_with(
        activity_token
    ).should.throw(SWFResponseError, "Unknown activity, scheduledEventId = 5")


# RespondActivityTaskFailed endpoint
@mock_swf_deprecated
def test_respond_activity_task_failed():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    activity_token = conn.poll_for_activity_task(
        "test-domain", "activity-task-list")["taskToken"]

    resp = conn.respond_activity_task_failed(activity_token,
                                             reason="short reason",
                                             details="long details")
    resp.should.be.none

    resp = conn.get_workflow_execution_history(
        "test-domain", conn.run_id, "uid-abcd1234")
    resp["events"][-2]["eventType"].should.equal("ActivityTaskFailed")
    resp["events"][-2]["activityTaskFailedEventAttributes"].should.equal(
        {"reason": "short reason", "details": "long details",
         "scheduledEventId": 5, "startedEventId": 6}
    )


@mock_swf_deprecated
def test_respond_activity_task_completed_with_wrong_token():
    # NB: we just test ONE failure case for RespondActivityTaskFailed
    # because the safeguards are shared with RespondActivityTaskCompleted, so
    # no need to retest everything end-to-end.
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    conn.poll_for_activity_task("test-domain", "activity-task-list")
    conn.respond_activity_task_failed.when.called_with(
        "not-a-correct-token"
    ).should.throw(SWFResponseError, "Invalid token")


# RecordActivityTaskHeartbeat endpoint
@mock_swf_deprecated
def test_record_activity_task_heartbeat():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    activity_token = conn.poll_for_activity_task(
        "test-domain", "activity-task-list")["taskToken"]

    resp = conn.record_activity_task_heartbeat(activity_token)
    resp.should.equal({"cancelRequested": False})


@mock_swf_deprecated
def test_record_activity_task_heartbeat_with_wrong_token():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    conn.poll_for_activity_task(
        "test-domain", "activity-task-list")["taskToken"]

    conn.record_activity_task_heartbeat.when.called_with(
        "bad-token", details="some progress details"
    ).should.throw(SWFResponseError)


@mock_swf_deprecated
def test_record_activity_task_heartbeat_sets_details_in_case_of_timeout():
    conn = setup_workflow()
    decision_token = conn.poll_for_decision_task(
        "test-domain", "queue")["taskToken"]
    conn.respond_decision_task_completed(decision_token, decisions=[
        SCHEDULE_ACTIVITY_TASK_DECISION
    ])
    with freeze_time("2015-01-01 12:00:00"):
        activity_token = conn.poll_for_activity_task(
            "test-domain", "activity-task-list")["taskToken"]
        conn.record_activity_task_heartbeat(
            activity_token, details="some progress details")

    with freeze_time("2015-01-01 12:05:30"):
        # => Activity Task Heartbeat timeout reached!!
        resp = conn.get_workflow_execution_history(
            "test-domain", conn.run_id, "uid-abcd1234")
        resp["events"][-2]["eventType"].should.equal("ActivityTaskTimedOut")
        attrs = resp["events"][-2]["activityTaskTimedOutEventAttributes"]
        attrs["details"].should.equal("some progress details")



from __future__ import unicode_literals

from botocore.exceptions import ClientError, ParamValidationError
import boto3
import sure  # noqa
from moto import mock_ec2, mock_kms, mock_rds2


@mock_rds2
def test_create_database():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBName='staging-postgres',
                                       DBInstanceClass='db.m1.small',
                                       LicenseModel='license-included',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=["my_sg"])
    database['DBInstance']['AllocatedStorage'].should.equal(10)
    database['DBInstance']['DBInstanceClass'].should.equal("db.m1.small")
    database['DBInstance']['LicenseModel'].should.equal("license-included")
    database['DBInstance']['MasterUsername'].should.equal("root")
    database['DBInstance']['DBSecurityGroups'][0][
        'DBSecurityGroupName'].should.equal('my_sg')
    database['DBInstance']['DBInstanceArn'].should.equal(
        'arn:aws:rds:us-west-2:1234567890:db:db-master-1')
    database['DBInstance']['DBInstanceStatus'].should.equal('available')
    database['DBInstance']['DBName'].should.equal('staging-postgres')
    database['DBInstance']['DBInstanceIdentifier'].should.equal("db-master-1")


@mock_rds2
def test_stop_database():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBName='staging-postgres',
                                       DBInstanceClass='db.m1.small',
                                       LicenseModel='license-included',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=["my_sg"])
    mydb = conn.describe_db_instances(DBInstanceIdentifier=database['DBInstance']['DBInstanceIdentifier'])['DBInstances'][0]
    mydb['DBInstanceStatus'].should.equal('available')
    # test stopping database should shutdown
    response = conn.stop_db_instance(DBInstanceIdentifier=mydb['DBInstanceIdentifier'])
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    response['DBInstance']['DBInstanceStatus'].should.equal('shutdown')
    # test rdsclient error when trying to stop an already stopped database
    conn.stop_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier']).should.throw(ClientError)
    # test stopping a stopped database with snapshot should error and no snapshot should exist for that call
    conn.stop_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier'], DBSnapshotIdentifier='rocky4570-rds-snap').should.throw(ClientError)
    response = conn.describe_db_snapshots()
    response['DBSnapshots'].should.equal([])


@mock_rds2
def test_start_database():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBName='staging-postgres',
                                       DBInstanceClass='db.m1.small',
                                       LicenseModel='license-included',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=["my_sg"])
    mydb = conn.describe_db_instances(DBInstanceIdentifier=database['DBInstance']['DBInstanceIdentifier'])['DBInstances'][0]
    mydb['DBInstanceStatus'].should.equal('available')
    # test starting an already started database should error
    conn.start_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier']).should.throw(ClientError)
    # stop and test start - should go from shutdown to available, create snapshot and check snapshot
    response = conn.stop_db_instance(DBInstanceIdentifier=mydb['DBInstanceIdentifier'], DBSnapshotIdentifier='rocky4570-rds-snap')
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    response['DBInstance']['DBInstanceStatus'].should.equal('shutdown')
    response = conn.describe_db_snapshots()
    response['DBSnapshots'][0]['DBSnapshotIdentifier'].should.equal('rocky4570-rds-snap')
    response = conn.start_db_instance(DBInstanceIdentifier=mydb['DBInstanceIdentifier'])
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    response['DBInstance']['DBInstanceStatus'].should.equal('available')
    # starting database should not remove snapshot
    response = conn.describe_db_snapshots()
    response['DBSnapshots'][0]['DBSnapshotIdentifier'].should.equal('rocky4570-rds-snap')
    # test stopping database, create snapshot with existing snapshot already created should throw error
    conn.stop_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier'], DBSnapshotIdentifier='rocky4570-rds-snap').should.throw(ClientError)
    # test stopping database not invoking snapshot should succeed.
    response = conn.stop_db_instance(DBInstanceIdentifier=mydb['DBInstanceIdentifier'])
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    response['DBInstance']['DBInstanceStatus'].should.equal('shutdown')


@mock_rds2
def test_fail_to_stop_multi_az():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBName='staging-postgres',
                                       DBInstanceClass='db.m1.small',
                                       LicenseModel='license-included',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=["my_sg"],
                                       MultiAZ=True)

    mydb = conn.describe_db_instances(DBInstanceIdentifier=database['DBInstance']['DBInstanceIdentifier'])['DBInstances'][0]
    mydb['DBInstanceStatus'].should.equal('available')
    # multi-az databases arent allowed to be shutdown at this time.
    conn.stop_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier']).should.throw(ClientError)
    # multi-az databases arent allowed to be started up at this time.
    conn.start_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier']).should.throw(ClientError)


@mock_rds2
def test_fail_to_stop_readreplica():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBName='staging-postgres',
                                       DBInstanceClass='db.m1.small',
                                       LicenseModel='license-included',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=["my_sg"])

    replica = conn.create_db_instance_read_replica(DBInstanceIdentifier="db-replica-1",
                                                   SourceDBInstanceIdentifier="db-master-1",
                                                   DBInstanceClass="db.m1.small")

    mydb = conn.describe_db_instances(DBInstanceIdentifier=replica['DBInstance']['DBInstanceIdentifier'])['DBInstances'][0]
    mydb['DBInstanceStatus'].should.equal('available')
    # read-replicas are not allowed to be stopped at this time.
    conn.stop_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier']).should.throw(ClientError)
    # read-replicas are not allowed to be started at this time.
    conn.start_db_instance.when.called_with(DBInstanceIdentifier=mydb['DBInstanceIdentifier']).should.throw(ClientError)


@mock_rds2
def test_get_databases():
    conn = boto3.client('rds', region_name='us-west-2')

    instances = conn.describe_db_instances()
    list(instances['DBInstances']).should.have.length_of(0)

    conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                            AllocatedStorage=10,
                            DBInstanceClass='postgres',
                            Engine='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=['my_sg'])
    conn.create_db_instance(DBInstanceIdentifier='db-master-2',
                            AllocatedStorage=10,
                            DBInstanceClass='postgres',
                            Engine='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=['my_sg'])
    instances = conn.describe_db_instances()
    list(instances['DBInstances']).should.have.length_of(2)

    instances = conn.describe_db_instances(DBInstanceIdentifier="db-master-1")
    list(instances['DBInstances']).should.have.length_of(1)
    instances['DBInstances'][0][
        'DBInstanceIdentifier'].should.equal("db-master-1")
    instances['DBInstances'][0]['DBInstanceArn'].should.equal(
        'arn:aws:rds:us-west-2:1234567890:db:db-master-1')


@mock_rds2
def test_get_databases_paginated():
    conn = boto3.client('rds', region_name="us-west-2")

    for i in range(51):
        conn.create_db_instance(AllocatedStorage=5,
                                Port=5432,
                                DBInstanceIdentifier='rds%d' % i,
                                DBInstanceClass='db.t1.micro',
                                Engine='postgres')

    resp = conn.describe_db_instances()
    resp["DBInstances"].should.have.length_of(50)
    resp["Marker"].should.equal(resp["DBInstances"][-1]['DBInstanceIdentifier'])

    resp2 = conn.describe_db_instances(Marker=resp["Marker"])
    resp2["DBInstances"].should.have.length_of(1)


@mock_rds2
def test_describe_non_existant_database():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.describe_db_instances.when.called_with(
        DBInstanceIdentifier="not-a-db").should.throw(ClientError)


@mock_rds2
def test_modify_db_instance():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       DBInstanceClass='postgres',
                                       Engine='db.m1.small',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=['my_sg'])
    instances = conn.describe_db_instances(DBInstanceIdentifier='db-master-1')
    instances['DBInstances'][0]['AllocatedStorage'].should.equal(10)
    conn.modify_db_instance(DBInstanceIdentifier='db-master-1',
                            AllocatedStorage=20,
                            ApplyImmediately=True)
    instances = conn.describe_db_instances(DBInstanceIdentifier='db-master-1')
    instances['DBInstances'][0]['AllocatedStorage'].should.equal(20)


@mock_rds2
def test_modify_non_existant_database():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.modify_db_instance.when.called_with(DBInstanceIdentifier='not-a-db',
                                             AllocatedStorage=20,
                                             ApplyImmediately=True).should.throw(ClientError)


@mock_rds2
def test_reboot_db_instance():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                            AllocatedStorage=10,
                            DBInstanceClass='postgres',
                            Engine='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=['my_sg'])
    database = conn.reboot_db_instance(DBInstanceIdentifier='db-master-1')
    database['DBInstance']['DBInstanceIdentifier'].should.equal("db-master-1")


@mock_rds2
def test_reboot_non_existant_database():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.reboot_db_instance.when.called_with(
        DBInstanceIdentifier="not-a-db").should.throw(ClientError)


@mock_rds2
def test_delete_database():
    conn = boto3.client('rds', region_name='us-west-2')
    instances = conn.describe_db_instances()
    list(instances['DBInstances']).should.have.length_of(0)
    conn.create_db_instance(DBInstanceIdentifier='db-primary-1',
                            AllocatedStorage=10,
                            Engine='postgres',
                            DBInstanceClass='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=['my_sg'])
    instances = conn.describe_db_instances()
    list(instances['DBInstances']).should.have.length_of(1)

    conn.delete_db_instance(DBInstanceIdentifier="db-primary-1",
                            FinalDBSnapshotIdentifier='primary-1-snapshot')

    instances = conn.describe_db_instances()
    list(instances['DBInstances']).should.have.length_of(0)

    # Saved the snapshot
    snapshots = conn.describe_db_snapshots(DBInstanceIdentifier="db-primary-1").get('DBSnapshots')
    snapshots[0].get('Engine').should.equal('postgres')


@mock_rds2
def test_delete_non_existant_database():
    conn = boto3.client('rds2', region_name="us-west-2")
    conn.delete_db_instance.when.called_with(
        DBInstanceIdentifier="not-a-db").should.throw(ClientError)


@mock_rds2
def test_create_db_snapshots():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_snapshot.when.called_with(
        DBInstanceIdentifier='db-primary-1',
        DBSnapshotIdentifier='snapshot-1').should.throw(ClientError)

    conn.create_db_instance(DBInstanceIdentifier='db-primary-1',
                            AllocatedStorage=10,
                            Engine='postgres',
                            DBName='staging-postgres',
                            DBInstanceClass='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=["my_sg"])

    snapshot = conn.create_db_snapshot(DBInstanceIdentifier='db-primary-1',
                                       DBSnapshotIdentifier='g-1').get('DBSnapshot')

    snapshot.get('Engine').should.equal('postgres')
    snapshot.get('DBInstanceIdentifier').should.equal('db-primary-1')
    snapshot.get('DBSnapshotIdentifier').should.equal('g-1')


@mock_rds2
def test_describe_db_snapshots():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_instance(DBInstanceIdentifier='db-primary-1',
                            AllocatedStorage=10,
                            Engine='postgres',
                            DBName='staging-postgres',
                            DBInstanceClass='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=["my_sg"])
    conn.describe_db_snapshots.when.called_with(
        DBInstanceIdentifier="db-primary-1").should.throw(ClientError)

    created = conn.create_db_snapshot(DBInstanceIdentifier='db-primary-1',
                                      DBSnapshotIdentifier='snapshot-1').get('DBSnapshot')

    created.get('Engine').should.equal('postgres')

    by_database_id = conn.describe_db_snapshots(DBInstanceIdentifier='db-primary-1').get('DBSnapshots')
    by_snapshot_id = conn.describe_db_snapshots(DBSnapshotIdentifier='snapshot-1').get('DBSnapshots')
    by_snapshot_id.should.equal(by_database_id)

    snapshot = by_snapshot_id[0]
    snapshot.should.equal(created)
    snapshot.get('Engine').should.equal('postgres')


@mock_rds2
def test_delete_db_snapshot():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_instance(DBInstanceIdentifier='db-primary-1',
                            AllocatedStorage=10,
                            Engine='postgres',
                            DBName='staging-postgres',
                            DBInstanceClass='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=["my_sg"])
    conn.create_db_snapshot(DBInstanceIdentifier='db-primary-1',
                            DBSnapshotIdentifier='snapshot-1')

    conn.describe_db_snapshots(DBSnapshotIdentifier='snapshot-1').get('DBSnapshots')[0]
    conn.delete_db_snapshot(DBSnapshotIdentifier='snapshot-1')
    conn.describe_db_snapshots.when.called_with(
        DBSnapshotIdentifier='snapshot-1').should.throw(ClientError)


@mock_rds2
def test_create_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    option_group = conn.create_option_group(OptionGroupName='test',
                                            EngineName='mysql',
                                            MajorEngineVersion='5.6',
                                            OptionGroupDescription='test option group')
    option_group['OptionGroup']['OptionGroupName'].should.equal('test')
    option_group['OptionGroup']['EngineName'].should.equal('mysql')
    option_group['OptionGroup'][
        'OptionGroupDescription'].should.equal('test option group')
    option_group['OptionGroup']['MajorEngineVersion'].should.equal('5.6')


@mock_rds2
def test_create_option_group_bad_engine_name():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group.when.called_with(OptionGroupName='test',
                                              EngineName='invalid_engine',
                                              MajorEngineVersion='5.6',
                                              OptionGroupDescription='test invalid engine').should.throw(ClientError)


@mock_rds2
def test_create_option_group_bad_engine_major_version():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group.when.called_with(OptionGroupName='test',
                                              EngineName='mysql',
                                              MajorEngineVersion='6.6.6',
                                              OptionGroupDescription='test invalid engine version').should.throw(ClientError)


@mock_rds2
def test_create_option_group_empty_description():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group.when.called_with(OptionGroupName='test',
                                              EngineName='mysql',
                                              MajorEngineVersion='5.6',
                                              OptionGroupDescription='').should.throw(ClientError)


@mock_rds2
def test_create_option_group_duplicate():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group(OptionGroupName='test',
                             EngineName='mysql',
                             MajorEngineVersion='5.6',
                             OptionGroupDescription='test option group')
    conn.create_option_group.when.called_with(OptionGroupName='test',
                                              EngineName='mysql',
                                              MajorEngineVersion='5.6',
                                              OptionGroupDescription='test option group').should.throw(ClientError)


@mock_rds2
def test_describe_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group(OptionGroupName='test',
                             EngineName='mysql',
                             MajorEngineVersion='5.6',
                             OptionGroupDescription='test option group')
    option_groups = conn.describe_option_groups(OptionGroupName='test')
    option_groups['OptionGroupsList'][0][
        'OptionGroupName'].should.equal('test')


@mock_rds2
def test_describe_non_existant_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.describe_option_groups.when.called_with(
        OptionGroupName="not-a-option-group").should.throw(ClientError)


@mock_rds2
def test_delete_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group(OptionGroupName='test',
                             EngineName='mysql',
                             MajorEngineVersion='5.6',
                             OptionGroupDescription='test option group')
    option_groups = conn.describe_option_groups(OptionGroupName='test')
    option_groups['OptionGroupsList'][0][
        'OptionGroupName'].should.equal('test')
    conn.delete_option_group(OptionGroupName='test')
    conn.describe_option_groups.when.called_with(
        OptionGroupName='test').should.throw(ClientError)


@mock_rds2
def test_delete_non_existant_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.delete_option_group.when.called_with(
        OptionGroupName='non-existant').should.throw(ClientError)


@mock_rds2
def test_describe_option_group_options():
    conn = boto3.client('rds', region_name='us-west-2')
    option_group_options = conn.describe_option_group_options(
        EngineName='sqlserver-ee')
    len(option_group_options['OptionGroupOptions']).should.equal(4)
    option_group_options = conn.describe_option_group_options(
        EngineName='sqlserver-ee', MajorEngineVersion='11.00')
    len(option_group_options['OptionGroupOptions']).should.equal(2)
    option_group_options = conn.describe_option_group_options(
        EngineName='mysql', MajorEngineVersion='5.6')
    len(option_group_options['OptionGroupOptions']).should.equal(1)
    conn.describe_option_group_options.when.called_with(
        EngineName='non-existent').should.throw(ClientError)
    conn.describe_option_group_options.when.called_with(
        EngineName='mysql', MajorEngineVersion='non-existent').should.throw(ClientError)


@mock_rds2
def test_modify_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group(OptionGroupName='test', EngineName='mysql',
                             MajorEngineVersion='5.6', OptionGroupDescription='test option group')
    # TODO: create option and validate before deleting.
    # if Someone can tell me how the hell to use this function
    # to add options to an option_group, I can finish coding this.
    result = conn.modify_option_group(OptionGroupName='test', OptionsToInclude=[
    ], OptionsToRemove=['MEMCACHED'], ApplyImmediately=True)
    result['OptionGroup']['EngineName'].should.equal('mysql')
    result['OptionGroup']['Options'].should.equal([])
    result['OptionGroup']['OptionGroupName'].should.equal('test')


@mock_rds2
def test_modify_option_group_no_options():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group(OptionGroupName='test', EngineName='mysql',
                             MajorEngineVersion='5.6', OptionGroupDescription='test option group')
    conn.modify_option_group.when.called_with(
        OptionGroupName='test').should.throw(ClientError)


@mock_rds2
def test_modify_non_existant_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.modify_option_group.when.called_with(OptionGroupName='non-existant', OptionsToInclude=[(
        'OptionName', 'Port', 'DBSecurityGroupMemberships', 'VpcSecurityGroupMemberships', 'OptionSettings')]).should.throw(ParamValidationError)


@mock_rds2
def test_delete_non_existant_database():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.delete_db_instance.when.called_with(
        DBInstanceIdentifier="not-a-db").should.throw(ClientError)


@mock_rds2
def test_list_tags_invalid_arn():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.list_tags_for_resource.when.called_with(
        ResourceName='arn:aws:rds:bad-arn').should.throw(ClientError)


@mock_rds2
def test_list_tags_db():
    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:db:foo')
    result['TagList'].should.equal([])
    test_instance = conn.create_db_instance(
        DBInstanceIdentifier='db-with-tags',
        AllocatedStorage=10,
        DBInstanceClass='postgres',
        Engine='db.m1.small',
        MasterUsername='root',
        MasterUserPassword='hunter2',
        Port=1234,
        DBSecurityGroups=['my_sg'],
        Tags=[
            {
                'Key': 'foo',
                'Value': 'bar',
            },
            {
                'Key': 'foo1',
                'Value': 'bar1',
            },
        ])
    result = conn.list_tags_for_resource(
        ResourceName=test_instance['DBInstance']['DBInstanceArn'])
    result['TagList'].should.equal([{'Value': 'bar',
                                     'Key': 'foo'},
                                    {'Value': 'bar1',
                                     'Key': 'foo1'}])


@mock_rds2
def test_add_tags_db():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_instance(DBInstanceIdentifier='db-without-tags',
                            AllocatedStorage=10,
                            DBInstanceClass='postgres',
                            Engine='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=['my_sg'],
                            Tags=[
                                {
                                    'Key': 'foo',
                                    'Value': 'bar',
                                },
                                {
                                    'Key': 'foo1',
                                    'Value': 'bar1',
                                },
                            ])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:db:db-without-tags')
    list(result['TagList']).should.have.length_of(2)
    conn.add_tags_to_resource(ResourceName='arn:aws:rds:us-west-2:1234567890:db:db-without-tags',
                              Tags=[
                                  {
                                      'Key': 'foo',
                                      'Value': 'fish',
                                  },
                                  {
                                      'Key': 'foo2',
                                      'Value': 'bar2',
                                  },
                              ])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:db:db-without-tags')
    list(result['TagList']).should.have.length_of(3)


@mock_rds2
def test_remove_tags_db():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_instance(DBInstanceIdentifier='db-with-tags',
                            AllocatedStorage=10,
                            DBInstanceClass='postgres',
                            Engine='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSecurityGroups=['my_sg'],
                            Tags=[
                                {
                                    'Key': 'foo',
                                    'Value': 'bar',
                                },
                                {
                                    'Key': 'foo1',
                                    'Value': 'bar1',
                                },
                            ])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:db:db-with-tags')
    list(result['TagList']).should.have.length_of(2)
    conn.remove_tags_from_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:db:db-with-tags', TagKeys=['foo'])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:db:db-with-tags')
    len(result['TagList']).should.equal(1)


@mock_rds2
def test_add_tags_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group(OptionGroupName='test',
                             EngineName='mysql',
                             MajorEngineVersion='5.6',
                             OptionGroupDescription='test option group')
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:og:test')
    list(result['TagList']).should.have.length_of(0)
    conn.add_tags_to_resource(ResourceName='arn:aws:rds:us-west-2:1234567890:og:test',
                              Tags=[
                                  {
                                      'Key': 'foo',
                                      'Value': 'fish',
                                  },
                                  {
                                      'Key': 'foo2',
                                      'Value': 'bar2',
                                  }])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:og:test')
    list(result['TagList']).should.have.length_of(2)


@mock_rds2
def test_remove_tags_option_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_option_group(OptionGroupName='test',
                             EngineName='mysql',
                             MajorEngineVersion='5.6',
                             OptionGroupDescription='test option group')
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:og:test')
    conn.add_tags_to_resource(ResourceName='arn:aws:rds:us-west-2:1234567890:og:test',
                              Tags=[
                                  {
                                      'Key': 'foo',
                                      'Value': 'fish',
                                  },
                                  {
                                      'Key': 'foo2',
                                      'Value': 'bar2',
                                  }])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:og:test')
    list(result['TagList']).should.have.length_of(2)
    conn.remove_tags_from_resource(ResourceName='arn:aws:rds:us-west-2:1234567890:og:test',
                                   TagKeys=['foo'])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:og:test')
    list(result['TagList']).should.have.length_of(1)


@mock_rds2
def test_create_database_security_group():
    conn = boto3.client('rds', region_name='us-west-2')

    result = conn.create_db_security_group(
        DBSecurityGroupName='db_sg', DBSecurityGroupDescription='DB Security Group')
    result['DBSecurityGroup']['DBSecurityGroupName'].should.equal("db_sg")
    result['DBSecurityGroup'][
        'DBSecurityGroupDescription'].should.equal("DB Security Group")
    result['DBSecurityGroup']['IPRanges'].should.equal([])


@mock_rds2
def test_get_security_groups():
    conn = boto3.client('rds', region_name='us-west-2')

    result = conn.describe_db_security_groups()
    result['DBSecurityGroups'].should.have.length_of(0)

    conn.create_db_security_group(
        DBSecurityGroupName='db_sg1', DBSecurityGroupDescription='DB Security Group')
    conn.create_db_security_group(
        DBSecurityGroupName='db_sg2', DBSecurityGroupDescription='DB Security Group')

    result = conn.describe_db_security_groups()
    result['DBSecurityGroups'].should.have.length_of(2)

    result = conn.describe_db_security_groups(DBSecurityGroupName="db_sg1")
    result['DBSecurityGroups'].should.have.length_of(1)
    result['DBSecurityGroups'][0]['DBSecurityGroupName'].should.equal("db_sg1")


@mock_rds2
def test_get_non_existant_security_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.describe_db_security_groups.when.called_with(
        DBSecurityGroupName="not-a-sg").should.throw(ClientError)


@mock_rds2
def test_delete_database_security_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_security_group(
        DBSecurityGroupName='db_sg', DBSecurityGroupDescription='DB Security Group')

    result = conn.describe_db_security_groups()
    result['DBSecurityGroups'].should.have.length_of(1)

    conn.delete_db_security_group(DBSecurityGroupName="db_sg")
    result = conn.describe_db_security_groups()
    result['DBSecurityGroups'].should.have.length_of(0)


@mock_rds2
def test_delete_non_existant_security_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.delete_db_security_group.when.called_with(
        DBSecurityGroupName="not-a-db").should.throw(ClientError)


@mock_rds2
def test_security_group_authorize():
    conn = boto3.client('rds', region_name='us-west-2')
    security_group = conn.create_db_security_group(DBSecurityGroupName='db_sg',
                                                   DBSecurityGroupDescription='DB Security Group')
    security_group['DBSecurityGroup']['IPRanges'].should.equal([])

    conn.authorize_db_security_group_ingress(DBSecurityGroupName='db_sg',
                                             CIDRIP='10.3.2.45/32')

    result = conn.describe_db_security_groups(DBSecurityGroupName="db_sg")
    result['DBSecurityGroups'][0]['IPRanges'].should.have.length_of(1)
    result['DBSecurityGroups'][0]['IPRanges'].should.equal(
        [{'Status': 'authorized', 'CIDRIP': '10.3.2.45/32'}])

    conn.authorize_db_security_group_ingress(DBSecurityGroupName='db_sg',
                                             CIDRIP='10.3.2.46/32')
    result = conn.describe_db_security_groups(DBSecurityGroupName="db_sg")
    result['DBSecurityGroups'][0]['IPRanges'].should.have.length_of(2)
    result['DBSecurityGroups'][0]['IPRanges'].should.equal([
        {'Status': 'authorized', 'CIDRIP': '10.3.2.45/32'},
        {'Status': 'authorized', 'CIDRIP': '10.3.2.46/32'},
    ])


@mock_rds2
def test_add_security_group_to_database():
    conn = boto3.client('rds', region_name='us-west-2')

    conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                            AllocatedStorage=10,
                            DBInstanceClass='postgres',
                            Engine='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234)

    result = conn.describe_db_instances()
    result['DBInstances'][0]['DBSecurityGroups'].should.equal([])
    conn.create_db_security_group(DBSecurityGroupName='db_sg',
                                  DBSecurityGroupDescription='DB Security Group')
    conn.modify_db_instance(DBInstanceIdentifier='db-master-1',
                            DBSecurityGroups=['db_sg'])
    result = conn.describe_db_instances()
    result['DBInstances'][0]['DBSecurityGroups'][0][
        'DBSecurityGroupName'].should.equal('db_sg')


@mock_rds2
def test_list_tags_security_group():
    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    security_group = conn.create_db_security_group(DBSecurityGroupName="db_sg",
                                                   DBSecurityGroupDescription='DB Security Group',
                                                   Tags=[{'Value': 'bar',
                                                          'Key': 'foo'},
                                                         {'Value': 'bar1',
                                                          'Key': 'foo1'}])['DBSecurityGroup']['DBSecurityGroupName']
    resource = 'arn:aws:rds:us-west-2:1234567890:secgrp:{0}'.format(
        security_group)
    result = conn.list_tags_for_resource(ResourceName=resource)
    result['TagList'].should.equal([{'Value': 'bar',
                                     'Key': 'foo'},
                                    {'Value': 'bar1',
                                     'Key': 'foo1'}])


@mock_rds2
def test_add_tags_security_group():
    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    security_group = conn.create_db_security_group(DBSecurityGroupName="db_sg",
                                                   DBSecurityGroupDescription='DB Security Group')['DBSecurityGroup']['DBSecurityGroupName']

    resource = 'arn:aws:rds:us-west-2:1234567890:secgrp:{0}'.format(
        security_group)
    conn.add_tags_to_resource(ResourceName=resource,
                              Tags=[{'Value': 'bar',
                                     'Key': 'foo'},
                                    {'Value': 'bar1',
                                     'Key': 'foo1'}])

    result = conn.list_tags_for_resource(ResourceName=resource)
    result['TagList'].should.equal([{'Value': 'bar',
                                     'Key': 'foo'},
                                    {'Value': 'bar1',
                                     'Key': 'foo1'}])


@mock_rds2
def test_remove_tags_security_group():
    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    security_group = conn.create_db_security_group(DBSecurityGroupName="db_sg",
                                                   DBSecurityGroupDescription='DB Security Group',
                                                   Tags=[{'Value': 'bar',
                                                          'Key': 'foo'},
                                                         {'Value': 'bar1',
                                                          'Key': 'foo1'}])['DBSecurityGroup']['DBSecurityGroupName']

    resource = 'arn:aws:rds:us-west-2:1234567890:secgrp:{0}'.format(
        security_group)
    conn.remove_tags_from_resource(ResourceName=resource, TagKeys=['foo'])

    result = conn.list_tags_for_resource(ResourceName=resource)
    result['TagList'].should.equal([{'Value': 'bar1', 'Key': 'foo1'}])


@mock_ec2
@mock_rds2
def test_create_database_subnet_group():
    vpc_conn = boto3.client('ec2', 'us-west-2')
    vpc = vpc_conn.create_vpc(CidrBlock='10.0.0.0/16')['Vpc']
    subnet1 = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/24')['Subnet']
    subnet2 = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/26')['Subnet']

    subnet_ids = [subnet1['SubnetId'], subnet2['SubnetId']]
    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.create_db_subnet_group(DBSubnetGroupName='db_subnet',
                                         DBSubnetGroupDescription='my db subnet',
                                         SubnetIds=subnet_ids)
    result['DBSubnetGroup']['DBSubnetGroupName'].should.equal("db_subnet")
    result['DBSubnetGroup'][
        'DBSubnetGroupDescription'].should.equal("my db subnet")
    subnets = result['DBSubnetGroup']['Subnets']
    subnet_group_ids = [subnets[0]['SubnetIdentifier'],
                        subnets[1]['SubnetIdentifier']]
    list(subnet_group_ids).should.equal(subnet_ids)


@mock_ec2
@mock_rds2
def test_create_database_in_subnet_group():
    vpc_conn = boto3.client('ec2', 'us-west-2')
    vpc = vpc_conn.create_vpc(CidrBlock='10.0.0.0/16')['Vpc']
    subnet = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/24')['Subnet']

    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_subnet_group(DBSubnetGroupName='db_subnet1',
                                DBSubnetGroupDescription='my db subnet',
                                SubnetIds=[subnet['SubnetId']])
    conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                            AllocatedStorage=10,
                            Engine='postgres',
                            DBInstanceClass='db.m1.small',
                            MasterUsername='root',
                            MasterUserPassword='hunter2',
                            Port=1234,
                            DBSubnetGroupName='db_subnet1')
    result = conn.describe_db_instances(DBInstanceIdentifier='db-master-1')
    result['DBInstances'][0]['DBSubnetGroup'][
        'DBSubnetGroupName'].should.equal('db_subnet1')


@mock_ec2
@mock_rds2
def test_describe_database_subnet_group():
    vpc_conn = boto3.client('ec2', 'us-west-2')
    vpc = vpc_conn.create_vpc(CidrBlock='10.0.0.0/16')['Vpc']
    subnet = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/24')['Subnet']

    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_subnet_group(DBSubnetGroupName="db_subnet1",
                                DBSubnetGroupDescription='my db subnet',
                                SubnetIds=[subnet['SubnetId']])
    conn.create_db_subnet_group(DBSubnetGroupName='db_subnet2',
                                DBSubnetGroupDescription='my db subnet',
                                SubnetIds=[subnet['SubnetId']])

    resp = conn.describe_db_subnet_groups()
    resp['DBSubnetGroups'].should.have.length_of(2)

    subnets = resp['DBSubnetGroups'][0]['Subnets']
    subnets.should.have.length_of(1)

    list(conn.describe_db_subnet_groups(DBSubnetGroupName="db_subnet1")
         ['DBSubnetGroups']).should.have.length_of(1)

    conn.describe_db_subnet_groups.when.called_with(
        DBSubnetGroupName="not-a-subnet").should.throw(ClientError)


@mock_ec2
@mock_rds2
def test_delete_database_subnet_group():
    vpc_conn = boto3.client('ec2', 'us-west-2')
    vpc = vpc_conn.create_vpc(CidrBlock='10.0.0.0/16')['Vpc']
    subnet = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/24')['Subnet']

    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    conn.create_db_subnet_group(DBSubnetGroupName="db_subnet1",
                                DBSubnetGroupDescription='my db subnet',
                                SubnetIds=[subnet['SubnetId']])
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(1)

    conn.delete_db_subnet_group(DBSubnetGroupName="db_subnet1")
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    conn.delete_db_subnet_group.when.called_with(
        DBSubnetGroupName="db_subnet1").should.throw(ClientError)


@mock_ec2
@mock_rds2
def test_list_tags_database_subnet_group():
    vpc_conn = boto3.client('ec2', 'us-west-2')
    vpc = vpc_conn.create_vpc(CidrBlock='10.0.0.0/16')['Vpc']
    subnet = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/24')['Subnet']

    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    subnet = conn.create_db_subnet_group(DBSubnetGroupName="db_subnet1",
                                         DBSubnetGroupDescription='my db subnet',
                                         SubnetIds=[subnet['SubnetId']],
                                         Tags=[{'Value': 'bar',
                                                'Key': 'foo'},
                                               {'Value': 'bar1',
                                                'Key': 'foo1'}])['DBSubnetGroup']['DBSubnetGroupName']
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:subgrp:{0}'.format(subnet))
    result['TagList'].should.equal([{'Value': 'bar',
                                     'Key': 'foo'},
                                    {'Value': 'bar1',
                                     'Key': 'foo1'}])


@mock_ec2
@mock_rds2
def test_add_tags_database_subnet_group():
    vpc_conn = boto3.client('ec2', 'us-west-2')
    vpc = vpc_conn.create_vpc(CidrBlock='10.0.0.0/16')['Vpc']
    subnet = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/24')['Subnet']

    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    subnet = conn.create_db_subnet_group(DBSubnetGroupName="db_subnet1",
                                         DBSubnetGroupDescription='my db subnet',
                                         SubnetIds=[subnet['SubnetId']],
                                         Tags=[])['DBSubnetGroup']['DBSubnetGroupName']
    resource = 'arn:aws:rds:us-west-2:1234567890:subgrp:{0}'.format(subnet)

    conn.add_tags_to_resource(ResourceName=resource,
                              Tags=[{'Value': 'bar',
                                     'Key': 'foo'},
                                    {'Value': 'bar1',
                                     'Key': 'foo1'}])

    result = conn.list_tags_for_resource(ResourceName=resource)
    result['TagList'].should.equal([{'Value': 'bar',
                                     'Key': 'foo'},
                                    {'Value': 'bar1',
                                     'Key': 'foo1'}])


@mock_ec2
@mock_rds2
def test_remove_tags_database_subnet_group():
    vpc_conn = boto3.client('ec2', 'us-west-2')
    vpc = vpc_conn.create_vpc(CidrBlock='10.0.0.0/16')['Vpc']
    subnet = vpc_conn.create_subnet(
        VpcId=vpc['VpcId'], CidrBlock='10.1.0.0/24')['Subnet']

    conn = boto3.client('rds', region_name='us-west-2')
    result = conn.describe_db_subnet_groups()
    result['DBSubnetGroups'].should.have.length_of(0)

    subnet = conn.create_db_subnet_group(DBSubnetGroupName="db_subnet1",
                                         DBSubnetGroupDescription='my db subnet',
                                         SubnetIds=[subnet['SubnetId']],
                                         Tags=[{'Value': 'bar',
                                                'Key': 'foo'},
                                               {'Value': 'bar1',
                                                'Key': 'foo1'}])['DBSubnetGroup']['DBSubnetGroupName']
    resource = 'arn:aws:rds:us-west-2:1234567890:subgrp:{0}'.format(subnet)

    conn.remove_tags_from_resource(ResourceName=resource, TagKeys=['foo'])

    result = conn.list_tags_for_resource(ResourceName=resource)
    result['TagList'].should.equal([{'Value': 'bar1', 'Key': 'foo1'}])


@mock_rds2
def test_create_database_replica():
    conn = boto3.client('rds', region_name='us-west-2')

    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBInstanceClass='db.m1.small',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=["my_sg"])

    replica = conn.create_db_instance_read_replica(DBInstanceIdentifier="db-replica-1",
                                                   SourceDBInstanceIdentifier="db-master-1",
                                                   DBInstanceClass="db.m1.small")
    replica['DBInstance'][
        'ReadReplicaSourceDBInstanceIdentifier'].should.equal('db-master-1')
    replica['DBInstance']['DBInstanceClass'].should.equal('db.m1.small')
    replica['DBInstance']['DBInstanceIdentifier'].should.equal('db-replica-1')

    master = conn.describe_db_instances(DBInstanceIdentifier="db-master-1")
    master['DBInstances'][0]['ReadReplicaDBInstanceIdentifiers'].should.equal([
                                                                              'db-replica-1'])

    conn.delete_db_instance(
        DBInstanceIdentifier="db-replica-1", SkipFinalSnapshot=True)

    master = conn.describe_db_instances(DBInstanceIdentifier="db-master-1")
    master['DBInstances'][0][
        'ReadReplicaDBInstanceIdentifiers'].should.equal([])


@mock_rds2
@mock_kms
def test_create_database_with_encrypted_storage():
    kms_conn = boto3.client('kms', region_name='us-west-2')
    key = kms_conn.create_key(Policy='my RDS encryption policy',
                              Description='RDS encryption key',
                              KeyUsage='ENCRYPT_DECRYPT')

    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBInstanceClass='db.m1.small',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234,
                                       DBSecurityGroups=["my_sg"],
                                       StorageEncrypted=True,
                                       KmsKeyId=key['KeyMetadata']['KeyId'])

    database['DBInstance']['StorageEncrypted'].should.equal(True)
    database['DBInstance']['KmsKeyId'].should.equal(
        key['KeyMetadata']['KeyId'])


@mock_rds2
def test_create_db_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    db_parameter_group = conn.create_db_parameter_group(DBParameterGroupName='test',
                                                        DBParameterGroupFamily='mysql5.6',
                                                        Description='test parameter group')

    db_parameter_group['DBParameterGroup'][
        'DBParameterGroupName'].should.equal('test')
    db_parameter_group['DBParameterGroup'][
        'DBParameterGroupFamily'].should.equal('mysql5.6')
    db_parameter_group['DBParameterGroup'][
        'Description'].should.equal('test parameter group')


@mock_rds2
def test_create_db_instance_with_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    db_parameter_group = conn.create_db_parameter_group(DBParameterGroupName='test',
                                                        DBParameterGroupFamily='mysql5.6',
                                                        Description='test parameter group')

    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='mysql',
                                       DBInstanceClass='db.m1.small',
                                       DBParameterGroupName='test',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234)

    len(database['DBInstance']['DBParameterGroups']).should.equal(1)
    database['DBInstance']['DBParameterGroups'][0][
        'DBParameterGroupName'].should.equal('test')
    database['DBInstance']['DBParameterGroups'][0][
        'ParameterApplyStatus'].should.equal('in-sync')


@mock_rds2
def test_create_database_with_default_port():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='postgres',
                                       DBInstanceClass='db.m1.small',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       DBSecurityGroups=["my_sg"])
    database['DBInstance']['Endpoint']['Port'].should.equal(5432)


@mock_rds2
def test_modify_db_instance_with_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    database = conn.create_db_instance(DBInstanceIdentifier='db-master-1',
                                       AllocatedStorage=10,
                                       Engine='mysql',
                                       DBInstanceClass='db.m1.small',
                                       MasterUsername='root',
                                       MasterUserPassword='hunter2',
                                       Port=1234)

    len(database['DBInstance']['DBParameterGroups']).should.equal(1)
    database['DBInstance']['DBParameterGroups'][0][
        'DBParameterGroupName'].should.equal('default.mysql5.6')
    database['DBInstance']['DBParameterGroups'][0][
        'ParameterApplyStatus'].should.equal('in-sync')

    db_parameter_group = conn.create_db_parameter_group(DBParameterGroupName='test',
                                                        DBParameterGroupFamily='mysql5.6',
                                                        Description='test parameter group')
    conn.modify_db_instance(DBInstanceIdentifier='db-master-1',
                            DBParameterGroupName='test',
                            ApplyImmediately=True)

    database = conn.describe_db_instances(
        DBInstanceIdentifier='db-master-1')['DBInstances'][0]
    len(database['DBParameterGroups']).should.equal(1)
    database['DBParameterGroups'][0][
        'DBParameterGroupName'].should.equal('test')
    database['DBParameterGroups'][0][
        'ParameterApplyStatus'].should.equal('in-sync')


@mock_rds2
def test_create_db_parameter_group_empty_description():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_parameter_group.when.called_with(DBParameterGroupName='test',
                                                    DBParameterGroupFamily='mysql5.6',
                                                    Description='').should.throw(ClientError)


@mock_rds2
def test_create_db_parameter_group_duplicate():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_parameter_group(DBParameterGroupName='test',
                                   DBParameterGroupFamily='mysql5.6',
                                   Description='test parameter group')
    conn.create_db_parameter_group.when.called_with(DBParameterGroupName='test',
                                                    DBParameterGroupFamily='mysql5.6',
                                                    Description='test parameter group').should.throw(ClientError)


@mock_rds2
def test_describe_db_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_parameter_group(DBParameterGroupName='test',
                                   DBParameterGroupFamily='mysql5.6',
                                   Description='test parameter group')
    db_parameter_groups = conn.describe_db_parameter_groups(
        DBParameterGroupName='test')
    db_parameter_groups['DBParameterGroups'][0][
        'DBParameterGroupName'].should.equal('test')


@mock_rds2
def test_describe_non_existant_db_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    db_parameter_groups = conn.describe_db_parameter_groups(
        DBParameterGroupName='test')
    len(db_parameter_groups['DBParameterGroups']).should.equal(0)


@mock_rds2
def test_delete_db_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_parameter_group(DBParameterGroupName='test',
                                   DBParameterGroupFamily='mysql5.6',
                                   Description='test parameter group')
    db_parameter_groups = conn.describe_db_parameter_groups(
        DBParameterGroupName='test')
    db_parameter_groups['DBParameterGroups'][0][
        'DBParameterGroupName'].should.equal('test')
    conn.delete_db_parameter_group(DBParameterGroupName='test')
    db_parameter_groups = conn.describe_db_parameter_groups(
        DBParameterGroupName='test')
    len(db_parameter_groups['DBParameterGroups']).should.equal(0)


@mock_rds2
def test_modify_db_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_parameter_group(DBParameterGroupName='test',
                                   DBParameterGroupFamily='mysql5.6',
                                   Description='test parameter group')

    modify_result = conn.modify_db_parameter_group(DBParameterGroupName='test',
                                                   Parameters=[{
                                                       'ParameterName': 'foo',
                                                       'ParameterValue': 'foo_val',
                                                       'Description': 'test param',
                                                       'ApplyMethod': 'immediate'
                                                   }]
                                                   )

    modify_result['DBParameterGroupName'].should.equal('test')

    db_parameters = conn.describe_db_parameters(DBParameterGroupName='test')
    db_parameters['Parameters'][0]['ParameterName'].should.equal('foo')
    db_parameters['Parameters'][0]['ParameterValue'].should.equal('foo_val')
    db_parameters['Parameters'][0]['Description'].should.equal('test param')
    db_parameters['Parameters'][0]['ApplyMethod'].should.equal('immediate')


@mock_rds2
def test_delete_non_existant_db_parameter_group():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.delete_db_parameter_group.when.called_with(
        DBParameterGroupName='non-existant').should.throw(ClientError)


@mock_rds2
def test_create_parameter_group_with_tags():
    conn = boto3.client('rds', region_name='us-west-2')
    conn.create_db_parameter_group(DBParameterGroupName='test',
                                   DBParameterGroupFamily='mysql5.6',
                                   Description='test parameter group',
                                   Tags=[{
                                       'Key': 'foo',
                                       'Value': 'bar',
                                   }])
    result = conn.list_tags_for_resource(
        ResourceName='arn:aws:rds:us-west-2:1234567890:pg:test')
    result['TagList'].should.equal([{'Value': 'bar', 'Key': 'foo'}])



from __future__ import unicode_literals

import json
from six.moves.urllib.parse import urlencode
import re
import sure  # noqa

import moto.server as server

'''
Test the different server responses
'''


def test_cloudformation_server_get():
    backend = server.create_backend_app("cloudformation")
    stack_name = 'test stack'
    test_client = backend.test_client()
    template_body = {
        "Resources": {},
    }
    create_stack_resp = test_client.action_data("CreateStack", StackName=stack_name,
                                                TemplateBody=json.dumps(template_body))
    create_stack_resp.should.match(
        r"<CreateStackResponse>.*<CreateStackResult>.*<StackId>.*</StackId>.*</CreateStackResult>.*</CreateStackResponse>", re.DOTALL)
    stack_id_from_create_response = re.search(
        "<StackId>(.*)</StackId>", create_stack_resp).groups()[0]

    list_stacks_resp = test_client.action_data("ListStacks")
    stack_id_from_list_response = re.search(
        "<StackId>(.*)</StackId>", list_stacks_resp).groups()[0]

    stack_id_from_create_response.should.equal(stack_id_from_list_response)



from __future__ import unicode_literals
import random
from moto.core.responses import BaseResponse
from moto.ec2.utils import filters_from_querystring


class Subnets(BaseResponse):

    def create_subnet(self):
        vpc_id = self._get_param('VpcId')
        cidr_block = self._get_param('CidrBlock')
        availability_zone = self._get_param(
            'AvailabilityZone', if_none=random.choice(
                self.ec2_backend.describe_availability_zones()).name)
        subnet = self.ec2_backend.create_subnet(
            vpc_id,
            cidr_block,
            availability_zone,
        )
        template = self.response_template(CREATE_SUBNET_RESPONSE)
        return template.render(subnet=subnet)

    def delete_subnet(self):
        subnet_id = self._get_param('SubnetId')
        subnet = self.ec2_backend.delete_subnet(subnet_id)
        template = self.response_template(DELETE_SUBNET_RESPONSE)
        return template.render(subnet=subnet)

    def describe_subnets(self):
        subnet_ids = self._get_multi_param('SubnetId')
        filters = filters_from_querystring(self.querystring)
        subnets = self.ec2_backend.get_all_subnets(subnet_ids, filters)
        template = self.response_template(DESCRIBE_SUBNETS_RESPONSE)
        return template.render(subnets=subnets)

    def modify_subnet_attribute(self):
        subnet_id = self._get_param('SubnetId')
        map_public_ip = self._get_param('MapPublicIpOnLaunch.Value')
        self.ec2_backend.modify_subnet_attribute(subnet_id, map_public_ip)
        return MODIFY_SUBNET_ATTRIBUTE_RESPONSE


CREATE_SUBNET_RESPONSE = """
<CreateSubnetResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
  <requestId>7a62c49f-347e-4fc4-9331-6e8eEXAMPLE</requestId>
  <subnet>
    <subnetId>{{ subnet.id }}</subnetId>
    <state>pending</state>
    <vpcId>{{ subnet.vpc_id }}</vpcId>
    <cidrBlock>{{ subnet.cidr_block }}</cidrBlock>
    <availableIpAddressCount>251</availableIpAddressCount>
    <availabilityZone>{{ subnet.availability_zone }}</availabilityZone>
    <tagSet>
      {% for tag in subnet.get_tags() %}
        <item>
          <resourceId>{{ tag.resource_id }}</resourceId>
          <resourceType>{{ tag.resource_type }}</resourceType>
          <key>{{ tag.key }}</key>
          <value>{{ tag.value }}</value>
        </item>
      {% endfor %}
    </tagSet>
  </subnet>
</CreateSubnetResponse>"""

DELETE_SUBNET_RESPONSE = """
<DeleteSubnetResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
   <requestId>7a62c49f-347e-4fc4-9331-6e8eEXAMPLE</requestId>
   <return>true</return>
</DeleteSubnetResponse>"""

DESCRIBE_SUBNETS_RESPONSE = """
<DescribeSubnetsResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
  <requestId>7a62c49f-347e-4fc4-9331-6e8eEXAMPLE</requestId>
  <subnetSet>
    {% for subnet in subnets %}
      <item>
        <subnetId>{{ subnet.id }}</subnetId>
        <state>available</state>
        <vpcId>{{ subnet.vpc_id }}</vpcId>
        <cidrBlock>{{ subnet.cidr_block }}</cidrBlock>
        <availableIpAddressCount>251</availableIpAddressCount>
        <availabilityZone>{{ subnet.availability_zone }}</availabilityZone>
        <defaultForAz>{{ subnet.default_for_az }}</defaultForAz>
        <mapPublicIpOnLaunch>{{ subnet.map_public_ip_on_launch }}</mapPublicIpOnLaunch>
        <tagSet>
          {% for tag in subnet.get_tags() %}
            <item>
              <resourceId>{{ tag.resource_id }}</resourceId>
              <resourceType>{{ tag.resource_type }}</resourceType>
              <key>{{ tag.key }}</key>
              <value>{{ tag.value }}</value>
            </item>
          {% endfor %}
        </tagSet>
      </item>
    {% endfor %}
  </subnetSet>
</DescribeSubnetsResponse>"""

MODIFY_SUBNET_ATTRIBUTE_RESPONSE = """
<ModifySubnetAttributeResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
  <requestId>7a62c49f-347e-4fc4-9331-6e8eEXAMPLE</requestId>
  <return>true</return>
</ModifySubnetAttributeResponse>"""



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import boto
import boto3
import botocore.exceptions
from boto.exception import SQSError
from boto.sqs.message import RawMessage, Message

import base64
import sure  # noqa
import time

from moto import settings, mock_sqs, mock_sqs_deprecated
from tests.helpers import requires_boto_gte
import tests.backport_assert_raises  # noqa
from nose.tools import assert_raises


@mock_sqs
def test_create_fifo_queue_fail():
    sqs = boto3.client('sqs', region_name='us-east-1')

    try:
        sqs.create_queue(
            QueueName='test-queue',
            Attributes={
                'FifoQueue': 'true',
            }
        )
    except botocore.exceptions.ClientError as err:
        err.response['Error']['Code'].should.equal('InvalidParameterValue')
    else:
        raise RuntimeError('Should of raised InvalidParameterValue Exception')

@mock_sqs
def test_create_fifo_queue():
    sqs = boto3.client('sqs', region_name='us-east-1')
    resp = sqs.create_queue(
        QueueName='test-queue.fifo',
        Attributes={
            'FifoQueue': 'true',
        }
    )
    queue_url = resp['QueueUrl']

    response = sqs.get_queue_attributes(QueueUrl=queue_url)
    response['Attributes'].should.contain('FifoQueue')
    response['Attributes']['FifoQueue'].should.equal('true')



@mock_sqs
def test_create_queue():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    new_queue = sqs.create_queue(QueueName='test-queue')
    new_queue.should_not.be.none
    new_queue.should.have.property('url').should.contain('test-queue')

    queue = sqs.get_queue_by_name(QueueName='test-queue')
    queue.attributes.get('QueueArn').should_not.be.none
    queue.attributes.get('QueueArn').split(':')[-1].should.equal('test-queue')
    queue.attributes.get('QueueArn').split(':')[3].should.equal('us-east-1')
    queue.attributes.get('VisibilityTimeout').should_not.be.none
    queue.attributes.get('VisibilityTimeout').should.equal('30')


@mock_sqs
def test_get_inexistent_queue():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    sqs.get_queue_by_name.when.called_with(
        QueueName='nonexisting-queue').should.throw(botocore.exceptions.ClientError)


@mock_sqs
def test_message_send_without_attributes():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    queue = sqs.create_queue(QueueName="blah")
    msg = queue.send_message(
        MessageBody="derp"
    )
    msg.get('MD5OfMessageBody').should.equal(
        '58fd9edd83341c29f1aebba81c31e257')
    msg.shouldnt.have.key('MD5OfMessageAttributes')
    msg.get('ResponseMetadata', {}).get('RequestId').should.equal(
        '27daac76-34dd-47df-bd01-1f6e873584a0')
    msg.get('MessageId').should_not.contain(' \n')

    messages = queue.receive_messages()
    messages.should.have.length_of(1)


@mock_sqs
def test_message_send_with_attributes():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    queue = sqs.create_queue(QueueName="blah")
    msg = queue.send_message(
        MessageBody="derp",
        MessageAttributes={
            'timestamp': {
                'StringValue': '1493147359900',
                'DataType': 'Number',
            }
        }
    )
    msg.get('MD5OfMessageBody').should.equal(
        '58fd9edd83341c29f1aebba81c31e257')
    msg.get('MD5OfMessageAttributes').should.equal(
        '235c5c510d26fb653d073faed50ae77c')
    msg.get('ResponseMetadata', {}).get('RequestId').should.equal(
        '27daac76-34dd-47df-bd01-1f6e873584a0')
    msg.get('MessageId').should_not.contain(' \n')

    messages = queue.receive_messages()
    messages.should.have.length_of(1)


@mock_sqs
def test_message_with_complex_attributes():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    queue = sqs.create_queue(QueueName="blah")
    msg = queue.send_message(
        MessageBody="derp",
        MessageAttributes={
            'ccc': {'StringValue': 'testjunk', 'DataType': 'String'},
            'aaa': {'BinaryValue': b'\x02\x03\x04', 'DataType': 'Binary'},
            'zzz': {'DataType': 'Number', 'StringValue': '0230.01'},
            'öther_encodings': {'DataType': 'String', 'StringValue': 'T\xFCst'}
        }
    )
    msg.get('MD5OfMessageBody').should.equal(
        '58fd9edd83341c29f1aebba81c31e257')
    msg.get('MD5OfMessageAttributes').should.equal(
        '8ae21a7957029ef04146b42aeaa18a22')
    msg.get('ResponseMetadata', {}).get('RequestId').should.equal(
        '27daac76-34dd-47df-bd01-1f6e873584a0')
    msg.get('MessageId').should_not.contain(' \n')

    messages = queue.receive_messages()
    messages.should.have.length_of(1)


@mock_sqs
def test_send_message_with_unicode_characters():
    body_one = 'Héllo!😀'

    sqs = boto3.resource('sqs', region_name='us-east-1')
    queue = sqs.create_queue(QueueName="blah")
    msg = queue.send_message(MessageBody=body_one)

    messages = queue.receive_messages()
    message_body = messages[0].body

    message_body.should.equal(body_one)


@mock_sqs
def test_set_queue_attributes():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    queue = sqs.create_queue(QueueName="blah")

    queue.attributes['VisibilityTimeout'].should.equal("30")

    queue.set_attributes(Attributes={"VisibilityTimeout": "45"})
    queue.attributes['VisibilityTimeout'].should.equal("45")


@mock_sqs
def test_create_queues_in_multiple_region():
    west1_conn = boto3.client('sqs', region_name='us-west-1')
    west1_conn.create_queue(QueueName="blah")

    west2_conn = boto3.client('sqs', region_name='us-west-2')
    west2_conn.create_queue(QueueName="test-queue")

    list(west1_conn.list_queues()['QueueUrls']).should.have.length_of(1)
    list(west2_conn.list_queues()['QueueUrls']).should.have.length_of(1)

    if settings.TEST_SERVER_MODE:
        base_url = 'http://localhost:5000'
    else:
        base_url = 'https://us-west-1.queue.amazonaws.com'

    west1_conn.list_queues()['QueueUrls'][0].should.equal(
        '{base_url}/123456789012/blah'.format(base_url=base_url))


@mock_sqs
def test_get_queue_with_prefix():
    conn = boto3.client("sqs", region_name='us-west-1')
    conn.create_queue(QueueName="prefixa-queue")
    conn.create_queue(QueueName="prefixb-queue")
    conn.create_queue(QueueName="test-queue")

    conn.list_queues()['QueueUrls'].should.have.length_of(3)

    queue = conn.list_queues(QueueNamePrefix="test-")['QueueUrls']
    queue.should.have.length_of(1)

    if settings.TEST_SERVER_MODE:
        base_url = 'http://localhost:5000'
    else:
        base_url = 'https://us-west-1.queue.amazonaws.com'

    queue[0].should.equal(
        "{base_url}/123456789012/test-queue".format(base_url=base_url))


@mock_sqs
def test_delete_queue():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    conn = boto3.client("sqs", region_name='us-east-1')
    conn.create_queue(QueueName="test-queue",
                      Attributes={"VisibilityTimeout": "3"})
    queue = sqs.Queue('test-queue')

    conn.list_queues()['QueueUrls'].should.have.length_of(1)

    queue.delete()
    conn.list_queues().get('QueueUrls').should.equal(None)

    with assert_raises(botocore.exceptions.ClientError):
        queue.delete()


@mock_sqs
def test_set_queue_attribute():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    conn = boto3.client("sqs", region_name='us-east-1')
    conn.create_queue(QueueName="test-queue",
                      Attributes={"VisibilityTimeout": '3'})

    queue = sqs.Queue("test-queue")
    queue.attributes['VisibilityTimeout'].should.equal('3')

    queue.set_attributes(Attributes={"VisibilityTimeout": '45'})
    queue = sqs.Queue("test-queue")
    queue.attributes['VisibilityTimeout'].should.equal('45')


@mock_sqs
def test_send_receive_message_without_attributes():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    conn = boto3.client("sqs", region_name='us-east-1')
    conn.create_queue(QueueName="test-queue")
    queue = sqs.Queue("test-queue")

    body_one = 'this is a test message'
    body_two = 'this is another test message'

    queue.send_message(MessageBody=body_one)
    queue.send_message(MessageBody=body_two)

    messages = conn.receive_message(
        QueueUrl=queue.url, MaxNumberOfMessages=2)['Messages']

    message1 = messages[0]
    message2 = messages[1]

    message1['Body'].should.equal(body_one)
    message2['Body'].should.equal(body_two)

    message1.shouldnt.have.key('MD5OfMessageAttributes')
    message2.shouldnt.have.key('MD5OfMessageAttributes')


@mock_sqs
def test_send_receive_message_with_attributes():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    conn = boto3.client("sqs", region_name='us-east-1')
    conn.create_queue(QueueName="test-queue")
    queue = sqs.Queue("test-queue")

    body_one = 'this is a test message'
    body_two = 'this is another test message'

    queue.send_message(
        MessageBody=body_one,
        MessageAttributes={
            'timestamp': {
                'StringValue': '1493147359900',
                'DataType': 'Number',
            }
        }
    )

    queue.send_message(
        MessageBody=body_two,
        MessageAttributes={
            'timestamp': {
                'StringValue': '1493147359901',
                'DataType': 'Number',
            }
        }
    )

    messages = conn.receive_message(
        QueueUrl=queue.url, MaxNumberOfMessages=2)['Messages']

    message1 = messages[0]
    message2 = messages[1]

    message1.get('Body').should.equal(body_one)
    message2.get('Body').should.equal(body_two)

    message1.get('MD5OfMessageAttributes').should.equal('235c5c510d26fb653d073faed50ae77c')
    message2.get('MD5OfMessageAttributes').should.equal('994258b45346a2cc3f9cbb611aa7af30')


@mock_sqs
def test_send_receive_message_timestamps():
    sqs = boto3.resource('sqs', region_name='us-east-1')
    conn = boto3.client("sqs", region_name='us-east-1')
    conn.create_queue(QueueName="test-queue")
    queue = sqs.Queue("test-queue")

    queue.send_message(MessageBody="derp")
    messages = conn.receive_message(
        QueueUrl=queue.url, MaxNumberOfMessages=1)['Messages']

    message = messages[0]
    sent_timestamp = message.get('Attributes').get('SentTimestamp')
    approximate_first_receive_timestamp = message.get('Attributes').get('ApproximateFirstReceiveTimestamp')

    int.when.called_with(sent_timestamp).shouldnt.throw(ValueError)
    int.when.called_with(approximate_first_receive_timestamp).shouldnt.throw(ValueError)


@mock_sqs
def test_receive_messages_with_wait_seconds_timeout_of_zero():
    """
    test that zero messages is returned with a wait_seconds_timeout of zero,
    previously this created an infinite loop and nothing was returned
    :return:
    """

    sqs = boto3.resource('sqs', region_name='us-east-1')
    queue = sqs.create_queue(QueueName="blah")

    messages = queue.receive_messages(WaitTimeSeconds=0)
    messages.should.equal([])


@mock_sqs
def test_receive_messages_with_wait_seconds_timeout_of_negative_one():
    """
    test that zero messages is returned with a wait_seconds_timeout of negative 1
    :return:
    """

    sqs = boto3.resource('sqs', region_name='us-east-1')
    queue = sqs.create_queue(QueueName="blah")

    messages = queue.receive_messages(WaitTimeSeconds=-1)
    messages.should.equal([])


@mock_sqs_deprecated
def test_send_message_with_xml_characters():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    body_one = '< & >'

    queue.write(queue.new_message(body_one))

    messages = conn.receive_message(queue, number_messages=1)

    messages[0].get_body().should.equal(body_one)


@requires_boto_gte("2.28")
@mock_sqs_deprecated
def test_send_message_with_attributes():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    body = 'this is a test message'
    message = queue.new_message(body)
    BASE64_BINARY = base64.b64encode(b'binary value').decode('utf-8')
    message_attributes = {
        'test.attribute_name': {'data_type': 'String', 'string_value': 'attribute value'},
        'test.binary_attribute': {'data_type': 'Binary', 'binary_value': BASE64_BINARY},
        'test.number_attribute': {'data_type': 'Number', 'string_value': 'string value'}
    }
    message.message_attributes = message_attributes

    queue.write(message)

    messages = conn.receive_message(queue)

    messages[0].get_body().should.equal(body)

    for name, value in message_attributes.items():
        dict(messages[0].message_attributes[name]).should.equal(value)


@mock_sqs_deprecated
def test_send_message_with_delay():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    body_one = 'this is a test message'
    body_two = 'this is another test message'

    queue.write(queue.new_message(body_one), delay_seconds=3)
    queue.write(queue.new_message(body_two))

    queue.count().should.equal(1)

    messages = conn.receive_message(queue, number_messages=2)
    assert len(messages) == 1
    message = messages[0]
    assert message.get_body().should.equal(body_two)
    queue.count().should.equal(0)


@mock_sqs_deprecated
def test_send_large_message_fails():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    body_one = 'test message' * 200000
    huge_message = queue.new_message(body_one)

    queue.write.when.called_with(huge_message).should.throw(SQSError)


@mock_sqs_deprecated
def test_message_becomes_inflight_when_received():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=2)
    queue.set_message_class(RawMessage)

    body_one = 'this is a test message'
    queue.write(queue.new_message(body_one))
    queue.count().should.equal(1)

    messages = conn.receive_message(queue, number_messages=1)
    queue.count().should.equal(0)

    assert len(messages) == 1

    # Wait
    time.sleep(3)

    queue.count().should.equal(1)


@mock_sqs_deprecated
def test_receive_message_with_explicit_visibility_timeout():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    body_one = 'this is another test message'
    queue.write(queue.new_message(body_one))

    queue.count().should.equal(1)
    messages = conn.receive_message(
        queue, number_messages=1, visibility_timeout=0)

    assert len(messages) == 1

    # Message should remain visible
    queue.count().should.equal(1)


@mock_sqs_deprecated
def test_change_message_visibility():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=2)
    queue.set_message_class(RawMessage)

    body_one = 'this is another test message'
    queue.write(queue.new_message(body_one))

    queue.count().should.equal(1)
    messages = conn.receive_message(queue, number_messages=1)

    assert len(messages) == 1

    queue.count().should.equal(0)

    messages[0].change_visibility(2)

    # Wait
    time.sleep(1)

    # Message is not visible
    queue.count().should.equal(0)

    time.sleep(2)

    # Message now becomes visible
    queue.count().should.equal(1)

    messages = conn.receive_message(queue, number_messages=1)
    messages[0].delete()
    queue.count().should.equal(0)


@mock_sqs_deprecated
def test_message_attributes():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=2)
    queue.set_message_class(RawMessage)

    body_one = 'this is another test message'
    queue.write(queue.new_message(body_one))

    queue.count().should.equal(1)

    messages = conn.receive_message(queue, number_messages=1)
    queue.count().should.equal(0)

    assert len(messages) == 1

    message_attributes = messages[0].attributes

    assert message_attributes.get('ApproximateFirstReceiveTimestamp')
    assert int(message_attributes.get('ApproximateReceiveCount')) == 1
    assert message_attributes.get('SentTimestamp')
    assert message_attributes.get('SenderId')


@mock_sqs_deprecated
def test_read_message_from_queue():
    conn = boto.connect_sqs()
    queue = conn.create_queue('testqueue')
    queue.set_message_class(RawMessage)

    body = 'foo bar baz'
    queue.write(queue.new_message(body))
    message = queue.read(1)
    message.get_body().should.equal(body)


@mock_sqs_deprecated
def test_queue_length():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    queue.write(queue.new_message('this is a test message'))
    queue.write(queue.new_message('this is another test message'))
    queue.count().should.equal(2)


@mock_sqs_deprecated
def test_delete_message():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    queue.write(queue.new_message('this is a test message'))
    queue.write(queue.new_message('this is another test message'))
    queue.count().should.equal(2)

    messages = conn.receive_message(queue, number_messages=1)
    assert len(messages) == 1
    messages[0].delete()
    queue.count().should.equal(1)

    messages = conn.receive_message(queue, number_messages=1)
    assert len(messages) == 1
    messages[0].delete()
    queue.count().should.equal(0)


@mock_sqs_deprecated
def test_send_batch_operation():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)

    # See https://github.com/boto/boto/issues/831
    queue.set_message_class(RawMessage)

    queue.write_batch([
        ("my_first_message", 'test message 1', 0),
        ("my_second_message", 'test message 2', 0),
        ("my_third_message", 'test message 3', 0),
    ])

    messages = queue.get_messages(3)
    messages[0].get_body().should.equal("test message 1")

    # Test that pulling more messages doesn't break anything
    messages = queue.get_messages(2)


@requires_boto_gte("2.28")
@mock_sqs_deprecated
def test_send_batch_operation_with_message_attributes():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)
    queue.set_message_class(RawMessage)

    message_tuple = ("my_first_message", 'test message 1', 0, {
                     'name1': {'data_type': 'String', 'string_value': 'foo'}})
    queue.write_batch([message_tuple])

    messages = queue.get_messages()
    messages[0].get_body().should.equal("test message 1")

    for name, value in message_tuple[3].items():
        dict(messages[0].message_attributes[name]).should.equal(value)


@mock_sqs_deprecated
def test_delete_batch_operation():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=3)

    conn.send_message_batch(queue, [
        ("my_first_message", 'test message 1', 0),
        ("my_second_message", 'test message 2', 0),
        ("my_third_message", 'test message 3', 0),
    ])

    messages = queue.get_messages(2)
    queue.delete_message_batch(messages)

    queue.count().should.equal(1)


@mock_sqs_deprecated
def test_queue_attributes():
    conn = boto.connect_sqs('the_key', 'the_secret')

    queue_name = 'test-queue'
    visibility_timeout = 3

    queue = conn.create_queue(
        queue_name, visibility_timeout=visibility_timeout)

    attributes = queue.get_attributes()

    attributes['QueueArn'].should.look_like(
        'arn:aws:sqs:us-east-1:123456789012:%s' % queue_name)

    attributes['VisibilityTimeout'].should.look_like(str(visibility_timeout))

    attribute_names = queue.get_attributes().keys()
    attribute_names.should.contain('ApproximateNumberOfMessagesNotVisible')
    attribute_names.should.contain('MessageRetentionPeriod')
    attribute_names.should.contain('ApproximateNumberOfMessagesDelayed')
    attribute_names.should.contain('MaximumMessageSize')
    attribute_names.should.contain('CreatedTimestamp')
    attribute_names.should.contain('ApproximateNumberOfMessages')
    attribute_names.should.contain('ReceiveMessageWaitTimeSeconds')
    attribute_names.should.contain('DelaySeconds')
    attribute_names.should.contain('VisibilityTimeout')
    attribute_names.should.contain('LastModifiedTimestamp')
    attribute_names.should.contain('QueueArn')


@mock_sqs_deprecated
def test_change_message_visibility_on_invalid_receipt():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=1)
    queue.set_message_class(RawMessage)

    queue.write(queue.new_message('this is another test message'))
    queue.count().should.equal(1)
    messages = conn.receive_message(queue, number_messages=1)

    assert len(messages) == 1

    original_message = messages[0]

    queue.count().should.equal(0)

    time.sleep(2)

    queue.count().should.equal(1)

    messages = conn.receive_message(queue, number_messages=1)

    assert len(messages) == 1

    original_message.change_visibility.when.called_with(
        100).should.throw(SQSError)


@mock_sqs_deprecated
def test_change_message_visibility_on_visible_message():
    conn = boto.connect_sqs('the_key', 'the_secret')
    queue = conn.create_queue("test-queue", visibility_timeout=1)
    queue.set_message_class(RawMessage)

    queue.write(queue.new_message('this is another test message'))
    queue.count().should.equal(1)
    messages = conn.receive_message(queue, number_messages=1)

    assert len(messages) == 1

    original_message = messages[0]

    queue.count().should.equal(0)

    time.sleep(2)

    queue.count().should.equal(1)

    original_message.change_visibility.when.called_with(
        100).should.throw(SQSError)


@mock_sqs_deprecated
def test_purge_action():
    conn = boto.sqs.connect_to_region("us-east-1")

    queue = conn.create_queue('new-queue')
    queue.write(queue.new_message('this is another test message'))
    queue.count().should.equal(1)

    queue.purge()

    queue.count().should.equal(0)


@mock_sqs_deprecated
def test_delete_message_after_visibility_timeout():
    VISIBILITY_TIMEOUT = 1
    conn = boto.sqs.connect_to_region("us-east-1")
    new_queue = conn.create_queue(
        'new-queue', visibility_timeout=VISIBILITY_TIMEOUT)

    m1 = Message()
    m1.set_body('Message 1!')
    new_queue.write(m1)

    assert new_queue.count() == 1

    m1_retrieved = new_queue.read()

    time.sleep(VISIBILITY_TIMEOUT + 1)

    m1_retrieved.delete()

    assert new_queue.count() == 0
